#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import sys
import time
import signal
from multiprocessing import Process
from threading import Thread

import collector
import tracer
import logging

pyProc = None


def pyRunCtx(pyCmd):
    sys.argv[:] = pyCmd
    progname = pyCmd[0]
    sys.path.insert(0, os.path.dirname(progname))

    logging.info("vaitrace compile python code: %s" % progname)

    with open(progname, 'rb') as fp:
        code = compile(fp.read(), progname, 'exec')

    globs = {
        '__file__': progname,
        '__name__': '__main__',
        '__package__': None,
        '__cached__': None,
    }

    logging.info("vaitrace exec poython code: %s" % progname)

    exec(code, globs, None)


force_exit = False


def handler(signum, frame):
    global pyProc

    if pyProc.is_alive():
        pyProc.kill()
        logging.info("Killing process...")
        logging.info("Processing trace data, please wait...")
    else:
        logging.info("Processing trace data, please wait...")

    global force_exit
    if force_exit:
        logging.error("Force exit...")
        exit(-1)
    force_exit = True


def run(globalOptions: dict):
    global pyProc

    signal.signal(signal.SIGINT, handler)
    signal.signal(signal.SIGTERM, handler)

    options = globalOptions

    if options.get('cmdline_args').get('bypass', False):
        cmd = options.get('control').get('cmd')
        logging.info("Bypass vaitrace, just run cmd")

        pyCmd = options.get('control').get('cmd')
        pyRunCtx(pyCmd)
        exit(0)

    """Preparing"""
    tracer.prepare(options)
    tracer.start()

    """requirememt format: ["tracerName", "tracerName1", "hwInfo", ...]"""
    collector.prepare(options, tracer.getSourceRequirement())
    collector.start()

    """Start Running"""
    pyCmd = options.get('control').get('cmd')
    timeout = options.get('control').get('timeout')

    # pyRunCtx(pyCmd)
    pyProc = Thread(target=pyRunCtx, args=(pyCmd,))
    pyProc.start()

    options['control']['launcher'] = "python"
    options['control']['pid'] = os.getpid()
    options['control']['time'] = time.strftime(
        "%Y-%m-%d %H:%M:%S", time.localtime())

    if timeout <= 0:
        pyProc.join()
    else:
        while timeout > 0:
            time.sleep(1)
            timeout -= 1
            p = pyProc.is_alive()
            if p == False:
                break

    if timeout == 0:
        logging.info("vaitrace timeout, waiting for Python thread terminated")
    pyProc.join()
    collector.stop()
    tracer.stop()

    tracer.process(collector.getData())




# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from writer.parser.timelineUtil import *
from writer.parser.tracepointUtil import *
from writer.parser.statUtil import *
from functools import reduce


def get_funcname(data: dict) -> str:
    return


def cpuTaskPaired(start: vaiTimelineEvent, end: vaiTimelineEvent):
    if not (start.type == 'start' and end.type == 'done'):
        return None
    if end.ts < start.ts:
        return None
    if start.pid != end.pid:
        return None
    if start.get_info('subgraph') != end.get_info('subgraph'):
        return None

    pairedEvent = vaiTimelineEvent(
        start.startTime, start.pid, start.type, start.coreType, start.coreId, start.info)

    pairedEvent.type = 'period'
    pairedEvent.startTime = start.ts
    pairedEvent.endTime = end.ts
    pairedEvent.duration = end.ts - start.ts

    return pairedEvent


def convert_cpu_task(data):
    xmodel_info = select_trace(data, 'section', "XMODEL")
    cpu_subgraph_info = [subg for subg in xmodel_info if subg.get(
        "device", "").lower() == 'cpu']

    trace_data = select_trace_classes(data, ["cpu-task"])
    threads = {}
    pairedEvents = []
    for l in trace_data:
        event = tracepointEvent(l).toTimelineEvent()
        if event.coreType != "cpu-task":
            continue
        pid = event.pid
        threads.setdefault(pid, []).append(event)

    stack = []
    for pid in threads.keys():
        for e in threads[pid]:
            stack.append(e)
            for s in stack[::-1]:
                r = cpuTaskPaired(s, e)
                if r != None:
                    try:
                        stack.remove(e)
                        stack.remove(s)
                    except BaseException:
                        pass

                    pairedEvents.append(r)

    cpu_task_calls = statTable("CPU Tasks")
    for event in pairedEvents:
        if event.type != 'period':
            continue
        cpu_task_calls.add(event.get_info('subgraph'), event.duration)

    ret = []

    for i in cpu_task_calls.output():
        subgraph_name = i.split(',')[0]
        subg_op_desc = ""
        for s in cpu_subgraph_info:
            if s.get("subgraph_name", "") == subgraph_name:
                op_list = s.get("op_list", "").split('|')
                op_type = [op.split('@')[1]
                           for op in op_list if len(op.split('@')) == 2]

                subg_op_desc = reduce(lambda x, y: "%s|%s" % (x, y), op_type)

        subg_op_desc = reduce(lambda x, y: "%s|%s" %
                              (x, y), (set(subg_op_desc.split('|'))))
        ret.append("%s,%s,\n" % (i[0:-2], subg_op_desc))

    return ret



#!/usr/bin/python3
# -*- coding:utf-8 -*-

# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import sys
import re
import json
import logging
import tracer.tracerBase
from ctypes import *
from subprocess import Popen, PIPE

import vaitraceDefaults


class XIRTracer(tracer.tracerBase.Tracer):
    def __init__(self):
        super().__init__('XIR', source=['stdIO'], compatible={
            'machine': ["x86_64", "aarch64"]})
        """
        for xmodel
        [
          {
            'model': 'abs path to model',
            'graph': {} // struct of the model
          }
        
        ]
        """
        self.models = []

    def getGraphStruct(self, modelPath: str, kernelName="") -> dict:

        try:
            a = Popen(['./dump_graph', modelPath], stdout=PIPE)
            a.wait()

            g = str(a.stdout.read(), encoding='utf-8')
        except:
            #logging.error("Acquiring xir graph failed")
            g = "{}"

        return json.loads(g)

    def process(self, data, t_range=[]):
        xirGraph = self.getGraphStruct(self.xmodel)
        if len(xirGraph) > 0:
            self.models.append({'model': self.xmodel,
                                'graph': xirGraph})

    def start(self):
        super().start()

    def stop(self):
        super().stop()

    def prepare(self, options: dict, debug: bool):
        self.xmodel = options.get('xmodel', "")

    def getData(self):
        return self.models


tracer.tracerBase.register(XIRTracer())



#!/usr/bin/python3

# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import pickle
import gzip
import os
import sys

IN = sys.argv[1]

try:
    TYPE = sys.argv[2]
except:
    TYPE = None
try:
    OUT = sys.argv[3]
except:
    OUT = None

with gzip.open(IN, 'rb') as xatRaw:
    dataRaw = pickle.load(xatRaw)

    for k in dataRaw.keys():
        if TYPE == None:
            print("Usage: xatDump [input] [key | all]")
            print("Available Keys: ", dataRaw.keys())
            exit(0)
        if TYPE == 'all':
            pass
        else:
            if k.lower().find(TYPE.lower()) < 0:
                continue
        if len(dataRaw[k]) == 0:
            continue

        if type(dataRaw[k]) == list:
            print(*dataRaw[k])
        else:
            print(dataRaw[k])



# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time
import os
import math
import logging
import decimal
import json
import vaitraceSetting
from copy import deepcopy
from writer.parser.timelineUtil import *
from writer.parser.tracepointUtil import *
from writer.parser import xmodel_parser
import writer.parser.statUtil
from writer.parser.unitUtil import *


POINT_PER_SECOND = 100
WINDOW_LENGTH = 0.30


def ips_profile(dpu_timelines):
    """Add FPS data"""
    ips_ret = []
    eventFPS = []

    totalRunCnt = 0

    n_cores = 0

    for dpu in dpu_timelines:
        if dpu.len() == 0:
            continue
        n_cores = n_cores + 1
        for run in dpu.timeline:
            if run.type != 'period':
                continue

            batch = int(run.batch)

            totalRunCnt += batch
            eventFPS.append((run.startTime, batch))

    eventFPS.sort(key=lambda x: x[0])

    def eventFilter(events, start, end):
        ret = []
        for e in events:
            if e[0] > end:
                break
            elif e[0] < start:
                continue
            else:
                ret.append(e)
        return ret

    if len(eventFPS) < 20:
        return []
    else:
        start_t = eventFPS[0][0]
        end_t = eventFPS[-1][0]
        overall_ips = round(totalRunCnt / (end_t - start_t), 2)
        interval = 1 / POINT_PER_SECOND

        time_point = start_t

        while (time_point + WINDOW_LENGTH < end_t):
            eventsInWindow = eventFilter(
                eventFPS, time_point, time_point + WINDOW_LENGTH)
            if len(eventsInWindow) < 3:
                """Skip this point"""
                time_point += interval
                continue
            frames = 0
            for e in eventsInWindow:
                frames += e[1]
            time = WINDOW_LENGTH
            fps = round(frames / time, 1)
            ips_ret.append([round(time_point * 1000, 2), fps])
            time_point += interval

    logging.info("[Summary] DPU cores involved: %d" % n_cores)
    logging.info("[Summary] Total inferences: %d" % totalRunCnt)
    logging.info("[Summary] Overall inferences/second: %.2f" % overall_ips)

    return ips_ret


class dpu_ip():
    def __init__(self):
        self.core_num = 0
        self.cores = []

        # from [xdputil query]
        self.ip_version = None
        self.timestamp = None
        self.commit_id = None

    def __str__(self):
        return json.dumps(self.summary(), sort_keys=True, indent=4, separators=(',', ': '))

    def summary(self):
        ret = deepcopy(self.__dict__)

        ret["cores"] = []

        for c in self.cores:
            ret["cores"].append(deepcopy(c.__dict__))

        return ret

    def add_core(self, new_core):
        if self.check_unique(new_core.full_name):
            self.core_num = self.core_num + 1
            self.cores.append(new_core)

    def check_unique(self, cu_full_name):
        for c in self.cores:
            if cu_full_name == c.full_name:
                return False

        return True

    class dpu_ip_core():
        def __init__(self, cu_core_id, cu_device_id, cu_address, cu_name, cu_full_name, cu_fp, cu_hw_rt=0, cu_batch=1):
            self.core_id = int(cu_core_id)
            self.device_id = int(cu_device_id)
            self.address = cu_address
            self.name = cu_name
            self.full_name = cu_full_name
            self.fp = hex(int(cu_fp))
            self.batch = cu_batch

            # from [xdputil query]
            self.axilite_freq = None
            self.isa = None
            self.pl_freq = None
            self.aie_freq = None
            self.peak_ops = None
            self.arch = None
            self.type = None
            self.load_parallel = None
            self.save_parallel = None


class DPUEventParser():
    def __init__(self):
        self.vartThreads = {
            # pid: {runtime info}
        }
        self.xmodelInfo = {}
        self.dpu_timelines = []
        self.dpuIpInfo = []

    def getDpuAieClk(self, dpu_arch):

        idcode = '0x' + vaitraceSetting.getChipId().get('idcode',{})[-7:]
        mepll_base_addr_dict = {"0x4ca8093": 0xf70a0000, "0x4cd3093": 0xf6d10000,"0x4cc8093": 0xf6420000,
                "0x4cc0093": 0xf6300000, "0x4cd0093": 0xf6d10000, "0x4c98093": 0xf6d10000, "0x4c9b093": 0xf6d10000}
        mepll_base_addr =  mepll_base_addr_dict.get(idcode, 0)
        if mepll_base_addr == 0 :
            logging.warning("Not support %s aie clk " % hostname)
            return 1
        MEPLL_CTRL = mepll_base_addr + 0x100
        ME_CORE_REF_CTRL = mepll_base_addr + 0x138
        PMCPLL_CTRL = 0xF1260040
        NOCPLL_CTRL = 0xF1260050
        HSM0_REF_CTRL = 0xF1260148
        
        ref_clk = int(os.popen('cat /sys/kernel/debug/clk/ref_clk/clk_rate').read())
        hsm0_ref = int(os.popen(f"devmem {HSM0_REF_CTRL} 32").read(), 16)

        if (hsm0_ref & 0x7) == 0:
            n_p_pll = int(os.popen(f"devmem {PMCPLL_CTRL} 32").read(), 16)
        elif (hsm0_ref & 0x7) == 3:
            n_p_pll = int(os.popen(f"devmem {NOCPLL_CTRL} 32").read(), 16)
        else:
            print("get a error value from hsm0")

        clkoutdiv = math.pow(2, ((n_p_pll & 0x30000) >> 16))
        n_p_cla = (n_p_pll & 0xFF00) >> 8  # bit 15:8
        n_p_clk = ref_clk * n_p_cla / clkoutdiv
        hsm0_cal = (hsm0_ref & 0x3ff00) >> 8  # bit 17:8
        hsm0_clk = n_p_clk / hsm0_cal
        mepll = int(os.popen(f"devmem {MEPLL_CTRL} 32").read(), 16)
        mepll_cal = (mepll & 0xff00) >> 8  # bit 15:8
        me_clkoutdiv = math.pow(2, ((mepll & 0x30000) >> 16))

        me_core = int(os.popen(f"devmem {ME_CORE_REF_CTRL} 32").read(), 16)
        me_core_cal = (me_core & 0x3ff00) >> 8  # bit17:8
        mePLL_freq = hsm0_clk * mepll_cal / me_clkoutdiv / me_core_cal
        return mePLL_freq

    def getDpuRuntimeInfo(self, event, key):
        pid = event.pid
        subgraphName = self.vartThreads.get(pid, {}).get(key)
        return "%s" % subgraphName

    def get_info_from_xmodel(self, trace_name, trace_workload_g):
        for subg in self.xmodelInfo:
            info = self.xmodelInfo[subg]
            model_name = info["name"]
            if model_name != trace_name:
                continue
            model_workload = info["workload"]
            model_workload_g = to_Gi(int(model_workload))
            if math.fabs(float(trace_workload_g) - model_workload_g) < 0.02:
                return (info)
        return {}

    def parseDpuRuntimeEvent(self, event: vaiTimelineEvent):
        runtimeEventInfo = event.info
        subgraphName = "subgraph_unknown"
        batchSize = 1
        it = ""
        ot = ""

        pid = event.pid
        if pid not in self.vartThreads.keys():
            self.vartThreads.setdefault(pid, {"subgraph": None})
            self.vartThreads.setdefault(pid, {"batch": 1})

        runtimeEventList = runtimeEventInfo

        for key in runtimeEventList.keys():
            if key == 'subgraph':
                subgraphName = runtimeEventList.get(key, "subgraph")
            elif key == 'batch':
                batchSize = runtimeEventList.get(key, 1)
            elif key == 'op_num':
                op_num = runtimeEventList.get(key, 0)
            elif key == 'workload':
                workload = float(runtimeEventList.get(key, 0))
                workload = round(workload / 1000.0 / 1000.0 / 1000.0, 2)
            elif key == 'depth':
                depth = int(runtimeEventList.get(key, 0))
            elif key == 'it':
                it = runtimeEventList.get(key, None)
                if it.find('(') > 0:
                    it = it.split('(')[0]
            elif key == 'ot':
                ot = runtimeEventList.get(key, None)
                if ot.find('(') > 0:
                    ot = ot.split('(')[0]
            else:
                pass

        self.vartThreads[pid]["subgraph"] = subgraphName
        self.vartThreads[pid]["batch"] = batchSize
        self.vartThreads[pid]["workload"] = workload
        self.vartThreads[pid]["depth"] = depth
        self.vartThreads[pid]["it"] = it
        self.vartThreads[pid]["ot"] = ot

    def create_dpu_ip(self, data):
        self.dpu_ip = dpu_ip()

        dpu_controller_info = select_trace(data, "classname", "dpu-controller")
        dpu_core_num = len(dpu_controller_info)

        for i in dpu_controller_info:
            cu_core_id = int(i.get("cu_core_id", -1))
            cu_device_id = int(i.get("cu_device_id", -1))
            if (cu_core_id == -1) or (cu_device_id == -1):
                logging.error(
                    "Cannot find DPU id in dpu_controller_info, this trace is Error:")
                logging.error("[%s]" % i)
                continue

            cu_addr = i.get("cu_addr", 0)
            cu_name = i.get("cu_name", "DPU_%d" % dpu_controller_info.index(i))
            cu_full_name = i.get("cu_full_name", cu_name)
            cu_fingerprint = i.get("cu_fingerprint", 0)

            """self, cu_core_id, cu_address, cu_name, cu_full_name, cu_fp"""
            core = dpu_ip.dpu_ip_core(
                cu_core_id, cu_device_id, cu_addr, cu_name, cu_full_name, cu_fingerprint)
            self.dpu_ip.add_core(core)

    def timeline_dpucore_binding(self):
        for core in self.dpu_ip.cores:
            self.dpu_timelines[core.core_id].dpuCore = core

    def parse_trace(self, trace_data):
        """All trace record sort by time"""
        trace_data.sort(key=lambda x: float(x.get('ts')))

        """The first event must be 'dpu-runner'"""
        runtime_env_valid = {}

        for l in trace_data:
            event = tracepointEvent(l).toTimelineEvent()

            """Do Timeline Sync"""
            # TBD

            if event.coreType == "dpu-runner":
                self.parseDpuRuntimeEvent(event)
                runtime_env_valid[event.pid] = True
                continue

            if runtime_env_valid.get(event.pid, False) == False:
                continue

            """ Info get from runtime """
            event.batch = self.getDpuRuntimeInfo(event, "batch")
            event.op_num = 1
            event.subgraph = self.getDpuRuntimeInfo(event, "subgraph")
            event.workload = self.getDpuRuntimeInfo(event, "workload")
            event.it = self.getDpuRuntimeInfo(event, "it")
            event.ot = self.getDpuRuntimeInfo(event, "ot")
            event.depth = self.getDpuRuntimeInfo(event, "depth")

            """ Info get from xmodel """
            xmodel_i = self.get_info_from_xmodel(
                event.subgraph, event.workload)
            event.op_num = xmodel_i.get("op_num", event.op_num)

            """ Updata high precision workload """
            event.workload_raw = xmodel_i.get("workload", event.workload)

            event.load_io_img_size = xmodel_i.get("load_io_img_size", 0)
            event.load_io_para_size = xmodel_i.get("load_io_para_size", 0)
            event.save_io_size = xmodel_i.get("save_io_size", 0)
            #event.i_tensor_shape = xmodel_i.get("i_tensor_shape", "")
            #event.o_tensor_shape = xmodel_i.get("o_tensor_shape", "")

            if event.coreType == 'dpu-controller':
                self.dpu_timelines[event.coreId].add(event)

        """ Fix dpu ip [batch size] field """
        for dpu in self.dpu_timelines:
            if dpu.len() == 0:
                continue

            first_event = dpu.timeline[0]
            core_id = first_event.coreId
            batch = int(first_event.batch)
            self.dpu_ip.cores[core_id].batch = batch

    def dpu_ip_add_query_info(self, data):

        ip_info = select_trace(data, "type", "DPU")[0].get("info")

        self.dpu_ip.ip_version = ip_info.get(
            "DPU IP Spec", {}).get("IP version", "")
        self.dpu_ip.timestamp = ip_info.get(
            "DPU IP Spec", {}).get("generation timestamp", "")
        self.dpu_ip.commit_id = ip_info.get(
            "DPU IP Spec", {}).get("git commit id", "")

        for cu_query in ip_info.get("kernels", []):
            for cu in self.dpu_ip.cores:
                if cu_query.get("is_vivado_flow", False) != True:
                    if cu_query.get("cu_name", "") != cu.full_name:
                        continue

                # matched!
                cu_arch_txt = cu_query.get("DPU Arch").split("_")
                cu.arch, cu.isa, cu.type = cu_arch_txt[0], cu_arch_txt[1], cu_arch_txt[2]

                cu.load_parallel = int(cu_query.get("Load Parallel", 0))
                cu.save_parallel = int(cu_query.get("Save Parallel", 0))

                cu.axilite_freq = int(cu_query.get(
                    "XRT Frequency (MHz)", 0)) * 1000 * 1000
                cu.pl_freq = int(cu_query.get(
                    "DPU Frequency (MHz)", 0)) * 1000 * 1000

                if (cu.arch == "DPUCVDX8G") or (cu.arch == "DPUCV2DX8G"):
                    cu.aie_freq = self.getDpuAieClk(cu.arch)
                    if (cu.aie_freq == 1):
                        cu.peak_ops = 1
                        return

                if cu.arch == "DPUCZDX8G":  # mpsoc
                    macs = int(cu.type[1:])
                    ops = (macs * (cu.pl_freq))
                elif (cu.arch == "DPUCVDX8G") or (cu.arch == "DPUCV2DX8G"):  # versal-1
                    cpb_list = re.findall(r'C(\d+)', cu.type)
                    cpb_n = int(cpb_list[0])
                    batch_list = re.findall(r'B(\d+)', cu.type)
                    batch_n = int(batch_list[0])
                    cu_n_judge = cu.type.find("CU")
                    if cu_n_judge != -1:
                        reg_cu_n = re.compile(r'(?<=CU)\d+')
                        cu_list = reg_cu_n.findall(cu.type)
                        cu_n = int(cu_list[0])
                    else:
                        cu_n = 1

                    if cu.arch == "DPUCVDX8G":
                        base = 256
                    if cu.arch == "DPUCV2DX8G":
                        base = 512

                    ops = base * cpb_n * batch_n * cu_n * cu.aie_freq
                else:
                    raise RuntimeError("unkwon version dpu")

                cu.peak_ops = ops

        # print(self.dpu_ip)

    def parse(self, data, dpu_ip_info, options):
        fg_mode = (options.get('runmode') == 'debug')

        """Two types of event tracing data included: dpuRuntimeEvent & dpuControllerEvent"""
        cuRetData = {}

        """get xmodel info from: {'classname': 'subgraph_info', 'info_file': '/tmp/vaitrace_subgraph_info_15230', 'section': 'INFO'}"""
        subgraph_info = select_trace(data, 'classname', "subgraph_info")
        self.xmodelInfo = xmodel_parser.xmodel_get_info(subgraph_info)

        # Check [RELEASE::SPLIT] mode for .xmodel
        if fg_mode:
            level2_subg = 0
            empty_level2_subg = 0
            for subg in self.xmodelInfo.values():
                if subg.get('depth', 0) == 2:
                    level2_subg += 1
                if subg.get('mc_size', 0) == 0:
                    empty_level2_subg += 1
            if (level2_subg == empty_level2_subg):
                logging.error("This xmodel does not support fine grained profiling")

        """Prepare at most 8 DPU timelines"""
        self.dpu_timelines = createTimelines('DPU', 8, options)
        time_offset = options.get("time_offset", 0)
        time_limit = options.get("time_limit", float('inf'))

        data = select_trace_classes(data, ["dpu-runner", "dpu-controller"])
        self.create_dpu_ip(select_trace(data, 'section', "INFO"))
        self.timeline_dpucore_binding()
        self.dpu_ip_add_query_info(dpu_ip_info)

        self.parse_trace(select_trace(data, 'section', "TRACE"))

        return self.dpu_timelines

    def get_dpu_ip_summary(self):
        return self.dpu_ip.summary()

    def get_dpu_profile_summary(self, options, fmt="vtf"):
        """
        DPU Profile Summary Format:
        Kernel Name,Number Of Runs,CU Full Name,Minimum Time (ms),Average Time (ms),Maximum Time (ms),Workload(GOP),DPU Performance(GOP/s),Mem IO(MB),Mem Bandwidth(MB/s),
        """

        runmachine = options.get('run_machine', {})
        subgraph_stat_time = {}
        subgraph_stat_hwcnt = {}
        ret = []

        for dpu in self.dpu_timelines:
            if dpu.len() == 0:
                continue
            for run_event in dpu.timeline:
                if run_event.type != 'period':
                    continue

                dpu_id = run_event.coreId
                subgrap_name = run_event.subgraph.strip()
                workload_raw = run_event.workload_raw
                subg_idx = "%s|%s" % (subgrap_name, workload_raw)
                time = run_event.duration
                if runmachine == "aarch64":
                    hwcounter = run_event.hwcounter

                if (subg_idx in self.xmodelInfo):
                    pass

                subgraph_stat_time.setdefault(dpu_id, {}).setdefault(
                    subg_idx, []).append(time)

                if runmachine == "aarch64":
                    subgraph_stat_hwcnt.setdefault(dpu_id, {}).setdefault(
                            subg_idx, []).append(hwcounter)

        for core_id in subgraph_stat_time.keys():
            for subg_idx in subgraph_stat_time[core_id]:
                subg_name = subg_idx.split('|')[0]
                times = sorted(subgraph_stat_time[core_id][subg_idx])

                min_t = times[0] * 1000
                max_t = times[-1] * 1000
                avg_t = sum(times) / len(times) * 1000
                runs = len(times)

                dpu_core = self.dpu_ip.cores[core_id]
                cu_name = dpu_core.name
                batch = dpu_core.batch
                ctrl_freq = dpu_core.axilite_freq

                if runmachine == "aarch64":
                    hw_cnt_stat = subgraph_stat_hwcnt[core_id][subg_idx]
                    hw_counter = sum(hw_cnt_stat) / len(hw_cnt_stat)
                    hw_rt = hw_counter / ctrl_freq * 1000
                elif runmachine == "x86_64":
                    hw_rt = avg_t

                display_name = "%s:batch-%d" % (cu_name, batch)

                workload = self.xmodelInfo.get(subg_idx, {}).get("workload", 0)
                depth = self.xmodelInfo.get(subg_idx, {}).get("depth", 0)
                it = str(self.xmodelInfo.get(subg_idx, {}).get(
                    "i_tensor_shape", "")).replace(",", "_")
                ot = str(self.xmodelInfo.get(subg_idx, {}).get(
                    "o_tensor_shape", "")).replace(",", "_")

                if runmachine == "aarch64":
                    if (dpu_core.peak_ops == 1):
                        logging.warning("Effic is runtime ops!")
                    effic = (workload * batch / hw_rt) / (dpu_core.peak_ops) * 1000
                elif runmachine == "x86_64":
                    effic = 0

                performance = workload * batch * 1000 / hw_rt #hw_rt = ms

                load_img_size = int(self.xmodelInfo.get(
                    subg_idx, {}).get("load_io_img_size", 0))
                load_para_size = int(self.xmodelInfo.get(
                    subg_idx, {}).get("load_io_para_size", 0))
                rank_id = int(self.xmodelInfo.get(
                    subg_idx, {}).get("rank_id", 0))

                write_fm_size = batch * int(self.xmodelInfo.get(
                    subg_idx, {}).get("save_io_size", 0))

                read_fm_size = load_img_size * batch

                read_wb_size = load_para_size

                mem_io = read_fm_size + read_wb_size + write_fm_size
                mem_io_bw = mem_io / hw_rt * 1000  # to MB
                #print(subg_name, cu_name, min_t, avg_t, max_t, runs, workload, write_fm_size)

                inst_size = int(self.xmodelInfo.get(subg_idx, {}).get("mc_size", 0))

                prt_data_set = {
                    "subg_name": subg_name,
                    "runs": runs,
                    "display_name": display_name,
                    "depth": depth,
                    "min_t": uConv(min_t),
                    "avg_t": uConv(avg_t),
                    "max_t": uConv(max_t),
                    "workload": uConv(workload, "Gi"),
                    "effic": uConv(effic, "%", 1),
                    "read_wb_size": uConv(read_wb_size, "MB"),
                    "read_fm_size": uConv(read_fm_size, "MB"),
                    "write_fm_size": uConv(write_fm_size, "MB"),
                    "mem_io_bw": uConv(mem_io_bw, "MB"),
                    "mem_io": uConv(mem_io, "MB"),
                    "hw_rt": uConv(hw_rt),
                    "rank_id": rank_id,
                    "inst_size": inst_size
                }

                txt_fmt = "{subg_name},{runs},{display_name},{min_t},{avg_t},{max_t},{workload},{effic},{read_wb_size},{read_fm_size},{write_fm_size},{mem_io_bw},{hw_rt},{rank_id},\n"
                vtf_fmt = "{subg_name},{runs},{display_name},{min_t:.3f},{avg_t:.3f},{max_t:.3f},{workload:.3f},{effic:.3f},{mem_io:.3f},{mem_io_bw:.3f},\n"
                json_fmt = f"{subg_name},{runs},{cu_name},{batch},{min_t},{avg_t},{max_t},{workload},{effic},{read_wb_size},{read_fm_size},{write_fm_size},{mem_io_bw},{hw_rt},{depth},{it},{ot},{performance},{inst_size}\n"

                if (fmt == "txt"):
                    ret.append(txt_fmt.format(**prt_data_set))
                elif (fmt == "json"):
                    ret.append(json_fmt)
                else:
                    ret.append(vtf_fmt.format(**prt_data_set))

        return ret

    def get_dpu_ips(self):
        return ips_profile(self.dpu_timelines)



#!/usr/bin/python3
# -*- coding:utf-8 -*-

# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import sys
import os
import logging
import time
import math
import copy
import threading
from ctypes import *
import copy
import logging
import argparse
import json

def merge(a: dict, b: dict):
    if hasattr(a, "keys") and hasattr(b, "keys"):
        for kb in b.keys():
            if kb in a.keys():
                merge(a[kb], b[kb])
            else:
                a.update(b)
def record_data_factory(size=10):
    class APMRecord(Structure):
        _fields_ = [('time', c_double), ('data', c_longlong * size)]

        def output(self):
            data_cur = []
            list_data = []
            data_cur = [*self.data]
            list_data.append("NOC")
            list_data.append(self.time)
            for i in range(0, size):
                list_data.append(data_cur[i])
            return list_data
    data =  APMRecord()
    return data

class NOC:
    def __init__(self, mem_type="noc_nmu", path='/usr/lib/libnmuperf.so', enable=True):
        self.nmuLib = cdll.LoadLibrary(path)
        self.data = []
        self.base_addr = []
        self.location_name = []
        self.interval = 0
        self.npi_freq = 0
        self.act_period = 0
        self.record_data_len = 0
        self.started = False
        self.type = mem_type
        self.enabled = enable
        option = self.get_noc_nmu_info()
        nodes = self.parser_noc_nmu_info(option)
        self.base_addr = self.get_noc_nmu_addr(nodes)
        self.npi_base_addr = self.get_noc_npi_addr(nodes)
        self.noc_nmu_addr = (c_int * len(self.base_addr))(*self.base_addr)
        self.nmuLib.create_noc_nmu_instance(self.type.encode(),
                int(self.npi_freq),self.noc_nmu_addr,
                int(self.npi_base_addr),int(len(self.base_addr)))
        self.aie_run_thread = threading.Thread(target=self.aie_run_loop)

   
    def start(self, interval=0.01):
        if self.enabled == False:
            return
        self.started = True
        self.interval = interval
        self.nmuLib.start.restype = c_int
        self.aie_run_thread.start()
        return self.nmuLib.start(c_double(interval))

    def stop(self):

        struct_data_len = 10
        if self.enabled == False:
            return

        self.started = False
        self.aie_run_thread.join()
        self.nmuLib.stop()
        data = record_data_factory(len(self.base_addr) * 2)
        pd = pointer(data)

        while (self.nmuLib.pop_data(pd) == 0):
            self.data.append(copy.deepcopy(data))

        self.nmuLib.get_act_period.restype = c_double
        self.act_period = self.nmuLib.get_act_period()


        self.nmuLib.get_record_data_len.restype = c_int
        self.record_data_len = self.nmuLib.get_record_data_len()
    


    def transTimebase(self):
        if self.enabled == False:
            return
        for i in range(0, len(self.data)):
            for j in range(0, self.record_data_len):
                self.data[i].data[j] = int(
                    self.data[i].data[j] / self.act_period)

    def getData(self):
        return [d.output() for d in self.data]

    def printData(self, data):
        if self.enabled == False:
            return
        info = str()
        info += "TimeStamp: %.7f\n" % data.time

        read = [data.data[i]/self.interval/1000/1000 
                for i in range(0, 10) if i % 2 == 0]
        info += "Read Ports:  "
        for d in read:
            info += "%8.1f" % d

        info += " MB/s\n"

        write = [data.data[i]/self.interval/1000 /1000
                for i in range(0, 10) if i % 2 == 1]
        info += "Write Ports: "
        for d in write:
            info += "%8.1f" % d

        info += " MB/s\n  "

        return info

    def get_noc_nmu_info(self):
        default_conf_json = ""
        cmd_parser = argparse.ArgumentParser(prog="test")
        cmd_parser.add_argument("-c", dest="config", nargs='?',
                 default=default_conf_json, help="Specify the config file")
        args = cmd_parser.parse_args()
        cfg_path = args.config
        if cfg_path != "":
            try:
                cfg_file = open(cfg_path, 'rt')
                cfg = json.load(cfg_file)
                overlayOption = cfg.get('options', {})
                return overlayOption
            except:
                logging.error(f"Invalid config file: {cfg_path}")
                exit(-1)
                logging.info(f"Applying Config File {cfg_path}")
            else:
                cfg = options
                overlayOption = {}
        return option

    def parser_noc_nmu_info(self, option):
        nmuOption = option.get('tracer', {}).get('noc_perf', {})
        self.interval = nmuOption.get("interval",{})
        self.npi_freq = nmuOption.get("npi_freq",{})
        nodes = nmuOption.get("nodes", {})
        
        return nodes

    def get_noc_npi_addr(self, noc_nodes):
        for k in range(len(noc_nodes)):
            if("npi" == noc_nodes[k].get("type", {})):
                return int(noc_nodes[k].get("phy_address", {}), 16)

    def get_noc_nmu_addr(self, noc_nodes):
        noc_nmu_addr = []
        for k in range(len(noc_nodes)):
            if("nmu" == noc_nodes[k].get("type", {})):
                addr = noc_nodes[k].get("phy_address", {})
                noc_nmu_addr.append(int(addr, 16))
                self.location_name.append(noc_nodes[k].get("location_name", {}))
        if (len(noc_nmu_addr)) > 8:
            logging.error("The maximum of sample channals is 8!")
            exit(1)
        return noc_nmu_addr

    def aie_run_loop(self):
        while self.started:
            os.system("xdputil run  /usr/share/vitis_ai_library/models/resnet50/resnet50.xmodel  /usr/share/vitis_ai_library/models/resnet50/resnet50.xmodel")

    def toMB_s(self,r):
        return float(r) / 1000 / 1000

    def convert_to_json_format(self,raw_data,time_offset=0.0):
        len2 = len(raw_data[1]) - 1
        list_data = []
        cur_list_data = []
        cunt = 0
        for cunt in range(0,len2):
            for rr in raw_data:
                if rr[0] != "NOC":
                    continue
                r = rr[1:]
                if cunt == 0 :
                    timestamp = round((float(r[0]) - time_offset) * 1000, )
                    if timestamp < 0:
                        continue
                    cur_list_data.append(timestamp)
                else:
                    cur_list_data.append(round(self.toMB_s(r[cunt]), 2))
            
            list_data.append(cur_list_data)
            cur_list_data = []
        return list_data

    def create_json_file(self,json_data):
        json_name = []
        D = {}
        data = {
            "noc_nmu_timestamp": json_data[0]
        }

        for k in range(0, len(self.location_name)):
            json_name.append(self.location_name[k] + "_READ")
            json_name.append(self.location_name[k] + "_WRITE")

        for i in range(0,len(json_name)):
            D[json_name[i]] = json_data[i+1]
        
        merge(data, D) 
        json_data = json.dumps(data, indent=4, separators=(",", ":"))
        with open('noc_nmu_data.json','w') as json_file:
            json_file.write(json_data)


if __name__ == '__main__':
    noc = NOC()
    noc.start(noc.interval)
    time.sleep(1)
    noc.stop()
    noc.transTimebase()
    raw_data = noc.getData()
    data_cur = noc.convert_to_json_format(raw_data)
    noc.create_json_file(data_cur)
    




# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import csv
import time
import pickle
import sys
import os
import gzip
import logging
from writer.parser.vart_parser import *
from writer.parser.tracepointUtil import *
from writer.parser.statUtil import *
from writer.parser.pyfunc import *
from writer.parser.cppfunc import *

event_id = 1
start_id = 1
vtf_events = []
apm_events = []
nmu_events = []
nmu_info = {}
literal_pool = []
dpu_core_num = 0
PID = 0
GEN_TIME = 0
DEBUG_MODE = False
XRT_INFO = {}
TIMESYNC = {}
dpu_profile_summary = []
cppFuncSummary = []
pyFuncSummary = []

"""
EventID:uint64, StartID:uint64, Timestamp:double, bucketID:uint32, Type:int32, [, TYPE SPECIFIC FIELDS]

DPU EVENTS:
DPU_RUN -> EventID, StartID, Timestamp, bucketID:uint32, DPU_RUN, kernelName:MID, ThreadID:uint64, batch_size:uint32, input_tensor:MID, output_tensor:MID 
"""


class VART_VTF:
    def __init__(self, eid, sid, ts, bid, kn, tid, bs, it="", ot="", wl=0, op_num=0, ef=0):
        self.eid = int(eid)
        self.sid = int(sid)

        def time_to_ms(t_s):
            return t_s * 1000
        self.ts = float(time_to_ms(ts))
        self.bid = int(bid)
        self.kn = int(kn)
        self.tid = int(tid)
        self.bs = int(bs)
        self.it = it
        self.ot = ot
        self.wl = round(wl, 3)
        self.ef = round(ef, 2)
        self.opn = op_num
        self.rs = ""

    def __str__(self):
        mandatory = "%d,%d,%f,%d,VART_RUNNER,%d,%d,%d" % (
            self.eid, self.sid, self.ts, self.bid, self.kn, self.tid, self.bs)
        optional = str(self.wl) + ',' + str(self.ef) + ',' + str(self.opn) + \
            ',' + str(self.it) + ',' + str(self.ot) + ',' + str(self.rs) + ','
        return mandatory + ',' + optional + '\n'


def toVTF(e):
    global event_id

    bid = e.coreId + 1
    subgraph_index = literal_pool.index(e.subgraph)

    it_index = literal_pool.index(e.it)
    ot_index = literal_pool.index(e.ot)

    op_num = e.op_num

    """Unit of workload: GOPS"""
    workload = float(e.workload)

    """Unit of efficiency: GOPS/s"""
    time_dur = e.endTime - e.startTime
    efficiency = round(workload / time_dur, 2)

    vtf_event_satat = VART_VTF(
        event_id, 0, e.startTime, bid, subgraph_index, e.pid, e.batch, it_index, ot_index, workload, op_num, efficiency)
    start_id = event_id
    event_id = event_id + 1
    vtf_event_end = VART_VTF(event_id, start_id, e.endTime,
                             bid, subgraph_index, e.pid, e.batch, it_index, ot_index, workload, op_num, efficiency)
    event_id = event_id + 1

    return (vtf_event_satat, vtf_event_end)


def do_timeline_sync(xat, sync_base="xrt"):
    global TIMESYNC
    timesync = xat.get('timesync')
    timesync.pop("time_range")
    timeout = xat.get('cmd', {}).get('timeout')

    timeline_base = float('inf')
    for k in timesync.keys():
        if timesync[k] == 0:
            continue
        if timesync[k] < timeline_base:
            timeline_base = timesync[k]

    for k in timesync.keys():
        if timesync[k] == 0:
            continue
        timesync[k] -= timeline_base

    TIMESYNC = {"sync": timesync, "timeout": timeout}


def convert_dpu(raw_data, hwInfo, options):
    global literal_pool, dpu_core_num, vtf_events, dpu_profile_summary
    global TIMESYNC
    offset = TIMESYNC.get('sync', {}).get('vart')
    timeout = TIMESYNC.get('timeout', 0)

    runmode = options.get('control', {}).get('runmode')

    runmachine = options.get('control', {}).get('platform',{}).get('machine',{})
    dpu_parser = DPUEventParser()
    timelines = dpu_parser.parse(
        raw_data, hwInfo, {"time_offset": offset, "time_limit": timeout, "runmode": runmode})

    dpu_profile_summary = dpu_parser.get_dpu_profile_summary({"run_machine": runmachine})

    """extracting all strings"""
    for dpu in timelines:
        for event in dpu.timeline:
            literal_pool.append(event.subgraph)
            literal_pool.append(event.it)
            literal_pool.append(event.ot)
    literal_pool = list(set(literal_pool))
    literal_pool.insert(0, "RESERVED")

    for dpu_timeline in timelines:
        if len(dpu_timeline.timeline) == 0:
            continue
        dpu_core_num += 1
        for event in dpu_timeline.timeline:
            s, e = toVTF(event)
            vtf_events.append(s)
            vtf_events.append(e)

    vtf_events.sort(key=lambda x: x.ts)


def convert_xapm(apm_data, time_offset=0.0):
    global apm_events

    def toMB_s(r):
        return float(r) / 1000 / 1000

    for rr in apm_data:
        if not rr.startswith("APM "):
            continue
        r = rr.split()[1:]
        timestamp = round((float(r[0]) - time_offset) * 1000, 4)
        if timestamp < 0:
            continue
        s_1_r = round(toMB_s(r[1]), 2)
        s_1_w = round(toMB_s(r[2]), 2)
        s_2_r = round(toMB_s(r[3]), 2)
        s_2_w = round(toMB_s(r[4]), 2)
        s_3_r = round(toMB_s(r[5]), 2)
        s_3_w = round(toMB_s(r[6]), 2)
        s_4_r = round(toMB_s(r[7]), 2)
        s_4_w = round(toMB_s(r[8]), 2)
        s_5_r = round(toMB_s(r[9]), 2)
        s_5_w = round(toMB_s(r[10]), 2)
        apm_events.append([timestamp, s_1_r, s_1_w, s_2_r,
                           s_2_w, s_3_r, s_3_w, s_4_r, s_4_w, s_5_r, s_5_w])

def convert_nmu(nmu_data,options, time_offset=0.0):
    global nmu_events, nmu_info

    if options.get('noc_nmu', '') != True:
        return
    sample_data = []
    def toMB_s(r):
        return float(r) / 1000 / 1000

    nmu_info = nmu_data[0]
    sample_data = nmu_data[1]
    for rr in sample_data:
        if not rr.startswith("NMU "):
            continue
        r = rr.split()[1:]
        timestamp = round((float(r[0]) - time_offset) * 1000, 4)
        if timestamp < 0:
            continue
        s_1_r = round(toMB_s(r[1]), 2)
        s_1_w = round(toMB_s(r[2]), 2)
        s_2_r = round(toMB_s(r[3]), 2)
        s_2_w = round(toMB_s(r[4]), 2)
        s_3_r = round(toMB_s(r[5]), 2)
        s_3_w = round(toMB_s(r[6]), 2)
        s_4_r = round(toMB_s(r[7]), 2)
        s_4_w = round(toMB_s(r[8]), 2)
        s_5_r = round(toMB_s(r[9]), 2)
        s_5_w = round(toMB_s(r[10]), 2)
        nmu_events.append([timestamp, s_1_r, s_1_w, s_2_r,
                           s_2_w, s_3_r, s_3_w, s_4_r, s_4_w, s_5_r, s_5_w])


def output_nmu_trace(OUTPUT_PATH, options):
    if options.get('noc_nmu', False) != True:
        return
    port_name = []
    per_name = []
    f_name = "noc_nmu_trace.csv"
    npi_freq = nmu_info.get('npi_freq', 0)
    sample_interval = nmu_info.get('interval', 0)
    with open(os.path.join(OUTPUT_PATH, f_name), "w+t") as vtf:
        """HEADER"""
        vtf.writelines("NOC_NMU_INFO\n")
        test_info = [
            "interval,%s\n" % sample_interval,
            "npi_freq,%s\n" % npi_freq,
        ]
        vtf.writelines(test_info)
        vtf.writelines("\n")
        sample_nodes = nmu_info.get('nodes', [])
        for p in sample_nodes:
            if p.get('type',' ') == 'nmu':
                port_name.append(p.get('location_name', ''))
        title = "NOC_NMU Bandwith\n"
        vtf.write(title)
        for i in port_name:
            per_name.append(i + "_R")
            per_name.append(i + "_W")
        port_header = "sampletime," + ",".join(per_name) + ",\n"
        vtf.write(port_header)
        for d in nmu_events:
            item = "%.2f,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,\n" % (
                d[0], d[1], d[2], d[3], d[4], d[5], d[6], d[7], d[8], d[9], d[10])
            vtf.write(item)
        vtf.write('\n')

def output_vart_trace(OUTPUT_PATH):
    global PID, GEN_TIME, XRT_INFO
    xrt_version = XRT_INFO.get("runtime", {}).get(
        "build", {}).get("version", "0.0.0")
    cus = XRT_INFO.get("board", {}).get("compute_unit", {})
    f_name = "vart_trace.csv"

    with open(os.path.join(OUTPUT_PATH, f_name), "w+t") as vtf:
        """HEADER"""
        vtf.writelines("HEADER\n")
        header = [
            "VTF File Version,1.0\n",
            "VTF File Type,0\n",
            "PID,%d\n" % PID,
            "Generated on,%s\n" % GEN_TIME,
            "Resolution,ms\n",
            "Min Resolution,us\n",
            "Trace Version,1.0\n",
            "XRT Version,%s\n" % xrt_version,
            "Tool Version,2020.1\n"
        ]
        vtf.writelines(header)
        vtf.writelines("\n")

        """STRUCTURE"""
        vtf.writelines("STRUCTURE\n")
        s = []
        s.append("Group_Start,DPU\n")
        for i in range(dpu_core_num):
            try:
                cu_name = cus[str(i)].get("name", "DPU_%d" % i)
            except:
                cu_name = "DPU_%d" % i
            s.append("Dynamic_Row, %d, %s\n" %
                     (i+1, cu_name))

        s.append("Group_End,DPU\n")
        s.append("\n")
        vtf.writelines(s)

        """MAPPING"""
        vtf.writelines("MAPPING\n")
        for item in literal_pool[1:]:
            mapping = "%d,%s\n" % (literal_pool.index(item), item)
            vtf.writelines(mapping)
        vtf.writelines("\n")

        """EVENTS"""
        vtf.writelines("EVENTS\n")
        for e in vtf_events:
            vtf.writelines(str(e))

    logging.info("[%s]: Events number: %d" % (f_name, len(vtf_events)))

# Statistical information for DPU kernels(min/max time etc.)


def output_profile_summary(OUTPUT_PATH):
    f_name = "profile_summary.csv"
    xrt_version = XRT_INFO.get("runtime", {}).get(
        "build", {}).get("version", "0.0.0").strip()
    xrt_branch = XRT_INFO.get("runtime", {}).get(
        "build", {}).get("branch", "N/A").strip()
    xrt_hash = XRT_INFO.get("runtime", {}).get(
        "build", {}).get("hash", "N/A").strip()
    xrt_date = XRT_INFO.get("runtime", {}).get(
        "build", {}).get("date", "N/A").strip()
    xrt_device = XRT_INFO.get("board", {}).get(
        "info", {}).get("dsa_name", "N/A").strip()
    cus = XRT_INFO.get("board", {}).get("compute_unit", {})
    global dpuLatencyStat, GEN_TIME, pyFuncSummary, cppFuncSummary

    with open(os.path.join(OUTPUT_PATH, f_name), "w+t") as csv_f:
        t = GEN_TIME
        HEADER = "Profile Summary\nGenerated on: %s\n" % t +\
                 "XRT build version: %s\n" % xrt_version +\
                 "Build version branch: %s\n" % xrt_branch +\
                 "Build version hash: %s\n" % xrt_hash +\
                 "Build version date: %s\n" % xrt_date +\
                 "Target devices: %s\n\n" % xrt_device
        csv_f.write(HEADER)

        title = "DPU Summary\n"
        column_headers = "Kernel Name,Number Of Runs,CU Full Name,Minimum Time (ms),Average Time (ms),Maximum Time (ms),Workload(GOP),DPU Performance(GOP/s),Mem IO(MB),Mem Bandwidth(MB/s),\n"
        csv_f.write(title)
        csv_f.write(column_headers)

        csv_f.writelines(dpu_profile_summary)

        csv_f.writelines(cppFuncSummary)
        csv_f.writelines(pyFuncSummary)
        csv_f.write('\n')


# DDR bandwidth and throughtput
def output_vitis_ai_profile(OUTPUT_PATH):
    f_name = "vitis_ai_profile.csv"
    global apm_events, DEBUG_MODE

    with open(os.path.join(OUTPUT_PATH, f_name), "w+t") as csv_f:
        title = "DPU Throughput\n"
        column_headers = "timestamp,FPS\n"
        csv_f.write(title)
        csv_f.write(column_headers)

        csv_f.write('\n')
        title = "DDR Bandwidth\n"
        column_headers = "timestamp,DDRC_PORT_S1_Read,DDRC_PORT_S1_Write,DDRC_PORT_S2_Read,DDRC_PORT_S2_Write,DDRC_PORT_S3_Read,DDRC_PORT_S3_Write,DDRC_PORT_S4_Read,DDRC_PORT_S4_Write,DDRC_PORT_S5_Read,DDRC_PORT_S5_Write,\n"
        csv_f.write(title)
        csv_f.write(column_headers)
        for d in apm_events:
            item = "%.2f,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,\n" % (
                d[0], d[1], d[2], d[3], d[4], d[5], d[6], d[7], d[8], d[9], d[10])
            csv_f.write(item)
        csv_f.write('\n')


def output(OUTPUT_PATH, options):
    output_vart_trace(OUTPUT_PATH)
    output_nmu_trace(OUTPUT_PATH, options)
    output_profile_summary(OUTPUT_PATH)
    output_vitis_ai_profile(OUTPUT_PATH)


def xat_to_vtf(xat, options):
    saveTo = "./"
    global PID, GEN_TIME, XRT_INFO, DEBUG_MODE, cppFuncSummary, pyFuncSummary
    PID = xat.get('cmd', {}).get('pid', 0)
    GEN_TIME = xat.get('cmd', {}).get('time', "0000-00-00 00:00:00")
    XRT_INFO = xat.get('xrt', {})
    runmode = xat.get('cmd', {}).get('runmode', "")
    if runmode == "debug":
        DEBUG_MODE = True

    xapm_ts_offset = xat.get('timesync', {}).get('vart', 0.0)
    do_timeline_sync(xat)
    convert_dpu(xat.get('vart'), xat.get('hwInfo'), options)
    convert_xapm(xat.get('xapm', {}), xapm_ts_offset)
    #convert_nmu(xat.get('nmu', {}), options, xapm_ts_offset)
    pyFuncSummary = convert_pyfunc(xat.get('pyfunc', {}))
    cppFuncSummary = convert_cppfunc(xat.get('function', {}))
    output(saveTo, options)




# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from writer.parser.tracepointUtil import *


def analyse_power(data):
    power_report = {}

    if len(data) == 0:
        return {}

    idle_power = min(data)
    peak_power = max(data)
    ave_power = sum(data) / len(data)

    power_report["idle_power"] = idle_power
    power_report["peak_power"] = peak_power
    power_report["ave_power"] = ave_power

    return power_report



# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .dpuMcParserBase import *


class dpuv4eInstParser(dpuMcParserBase):
    def __init__(self):
        name = "DPUCVDX8H_ISA1"
        super().__init__(name)
        self.opcode_table = [
            self.inst_desc("LOAD", 0b0000, 5),
            self.inst_desc("SAVE", 0b0100, 4),
            self.inst_desc("CONV", 0b1000, 5),
            self.inst_desc("CONVINIT", 0b1001, 5),
            self.inst_desc("DPTWISE", 0b1010, 5),
            self.inst_desc("DWINIT", 0b1011, 4),
            self.inst_desc("POOLINIT", 0b0110, 4),
            self.inst_desc("POOL", 0b1100, 5),
            self.inst_desc("ELEWINIT", 0b1101, 2),
            self.inst_desc("ELEW", 0b1110, 4),
            self.inst_desc("END", 0b0111, 1)
        ]

    class Load(LittleEndianStructure):
        _pack_ = 1
        _fields_ = [
            ('bank_addr', c_uint, 13),
            ('output_channel_num', c_uint, 7),
            ('dpby', c_uint, 4),
            ('dpdon', c_uint, 4),
            ('opcode', c_uint, 4),


            ('jump_read', c_uint, 16),
            ('bank_id', c_uint, 8),
            ('ddr_mode', c_uint, 1),
            ('pad_idx', c_uint, 5),
            ('r0', c_uint, 2),

            ('channel', c_uint, 12),
            ('mode_avg', c_uint, 2),
            ('length', c_uint, 10),
            ('jump_write', c_uint, 8),

            ('ddr_addr', c_uint, 29),
            ('reg_id', c_uint, 3),

            ('jump_read_endl', c_uint, 21),
            ('pad_end', c_uint, 6),
            ('pad_start', c_uint, 5)
        ]

    class Save(LittleEndianStructure):
        _pack_ = 1
        _fields_ = [
            ('bank_addr', c_uint, 13),
            ('r0', c_uint, 7),
            ('dpby', c_uint, 4),
            ('dpdon', c_uint, 4),
            ('opcode', c_uint, 4),

            ('jump_write', c_uint, 16),
            ('bank_id', c_uint, 8),
            ('r1', c_uint, 8),

            ('channel', c_uint, 12),
            ('r2', c_uint, 2),
            ('length', c_uint, 10),
            ('jump_read', c_uint, 8),

            ('ddr_addr', c_uint, 29),
            ('reg_id', c_uint, 3)
        ]

    def process(self, mc, _debug=False):
        self.debug = _debug

        pos = 0
        load_img_size = 0
        load_para_size = 0
        save_size = 0

        while (pos < len(mc)):
            inst_name, inst_len = self.get_inst(mc, pos, self)

            if self.debug:
                print("pos: %d/%d, inst: %s          " %
                      (pos, len(mc), inst_name), end="\r")
            if inst_name == "LOAD":
                l = self.Load()
                inst = mc[pos:]
                memmove(addressof(l), inst, inst_len)

                load_bank_id = l.bank_id
                load_len = l.length
                load_channel = l.channel + 1
                load_jr = l.jump_read+1

                inst_load_img_size = 0
                inst_load_para_size = 0

                if load_bank_id < 16:
                    inst_load_img_size = load_len * load_jr + load_channel
                    # print("LOAD_FM:bank_id:%d, load_jr:%d, load_len:%d, load_channel:%d, %d" % \
                    #        (load_bank_id, load_jr, load_len, load_channel, inst_load_img_size))
                else:
                    inst_load_para_size = l.output_channel_num * l.jump_read_endl
                    # print("LOAD_PARA:bank_id:%d, load_jr:%d, load_len:%d, load_channel:%d, %d" % \
                    #        (load_bank_id, load_jr, load_len, load_channel, inst_load_para_size))

                load_img_size += inst_load_img_size
                load_para_size += inst_load_para_size

            elif inst_name == "SAVE":
                s = self.Save()
                inst = mc[pos:]
                memmove(addressof(s), inst, inst_len)

                length_ = s.length + 1
                chan_ = s.channel + 1
                save_size += (length_ * chan_)
                #print("SAVE %d %d" % (length_, chan_))
            elif inst_name == "UNKNOW":
                load_img_size = 0
                load_para_size = 0
                save_size = 0
                break
            else:
                pass

            pos += inst_len

        self.data["load_img_size"] = load_img_size
        self.data["load_para_size"] = load_para_size
        self.data["save_size"] = save_size


register(dpuv4eInstParser())




# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
from tempfile import TemporaryFile
import collector


class stdIOCollector(collector.collectorBase.Collector):
    def __init__(self, stdout="", stderr=""):
        super().__init__(name='stdIO')

    def __del__(self):
        if self.stdoutLogger is not None:
            self.stdoutLogger.close()
        if self.stderrLogger is not None:
            self.stderrLogger.close()

    def prepare(self, conf: dict) -> dict:
        self.stdoutLogger = TemporaryFile('w+t')
        self.stderrLogger = TemporaryFile('w+t')

        logger = {
            'stdout': self.stdoutLogger,
            'stderr': self.stderrLogger
        }

        return logger

    def start(self):
        super().start()

    def stop(self):
        super().stop()

    def getData(self):
        self.stdoutLogger.flush()
        self.stdoutLogger.seek(0)

        self.stderrLogger.flush()
        self.stderrLogger.seek(0)

        return self.stdoutLogger.readlines() + self.stderrLogger.readlines()


collector.collectorBase.register(stdIOCollector())



#!/usr/bin/python3
# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import csv
import os
import sys
from collections import namedtuple


try:
    CSV_PATH = sys.argv[1]
except:
    CSV_PATH = "./test_dpu_mc.csv"

csv_f = open(CSV_PATH, "rt")
csv_reader = csv.reader(csv_f)
csv_header = next(csv_reader)

analysis_list = []
XModelRaw = namedtuple("xmodel_raw", csv_header)

# prepare data
for m in csv_reader:
    analysis_list.append(XModelRaw(*m))

result_list = []
XModelAnaRes = namedtuple(
    "xmodel_ana", ["raw", "total_read", "total_acc", "comp_density"])
# analysis
for m in analysis_list:
    total_read = int(m.load_img_size) + int(m.load_para_size)
    total_acc = int(m.save_size) + total_read
    workload = int(m.workload)
    den = workload / total_acc

    result_list.append(XModelAnaRes(m, total_read, total_acc, den))

# print result
result_list.sort(key=lambda x: x.comp_density)
for m in result_list:
    print("%-s:%-s:   %.3f" %
          (m.raw.xmodel, m.raw.subgraph_name, m.comp_density))

csv_f.close()




# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

class Collector:
    def __init__(self, name):
        self.numClient = 0
        self.running = 0
        self.name = name

    def getData(self):
        pass

    def prepare(self):
        pass

    def start(self):
        self.running = True

    def stop(self):
        self.running = False


__collectors = []
__activeCollectors = []


def getCollectorInstances():
    return __collectors


def getCollectorInstances(name: str):
    for c in __collectors:
        if match(name == c.name):
            return c

    return None


def getAvailableCollectors():
    return [c.name for c in __collectors]


def start():
    for c in __activeCollectors:
        c.start()


def stop():
    for c in __activeCollectors:
        c.stop()


"""
option: {
    collectorOption: {
        ftrace: {
            sched: [],
            cu: [],
            ...
        },
    }
}

"""


def prepare(conf: dict, requirements: list) -> dict:
    def match(req: str, collector):
        if req.lower() == collector.name.lower():
            return True
        return False

    for name in list(set(requirements)):
        for c in __collectors:
            if match(name, c):
                __activeCollectors.append(c)
                conf.update(c.prepare(conf))

    return conf


def getData(source=None) -> dict:
    data = dict()

    """Make sure all avtive has been stopped"""
    for c in __activeCollectors:
        if c.running == True:
            assert ()

    for c in __activeCollectors:
        data.update({c.name: c.getData()})

    return data


def register(collectorInstance):
    __collectors.append(collectorInstance)



#!/usr/bin/python3

# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import os
import ctypes
import logging
import json
from ascii_table import *

"""   MEMORY TOPOLOGY SECTION   """


class mem_u1 (ctypes.Union):
    _fields_ = [
        ("m_size", ctypes.c_int64),
        ("route_id", ctypes.c_int64)
    ]


class mem_u2 (ctypes.Union):
    _fields_ = [
        ("m_base_address", ctypes.c_int64),
        ("flow_id", ctypes.c_int64)
    ]


class mem_data (ctypes.Structure):
    _anonymous_ = ("mem_u1", "mem_u2")
    _fields_ = [
        ("m_type", ctypes.c_uint8),
        ("m_used", ctypes.c_uint8),
        ("mem_u1", mem_u1),
        ("mem_u2", mem_u2),
        ("m_tag", ctypes.c_char * 16)
    ]


class mem_topology (ctypes.Structure):
    _fields_ = [
        ("m_count", ctypes.c_int32),
        ("m_mem_data", mem_data*1000)
    ]


"""   IP_LAYOUT SECTION   """


class IP_CONTROL:
    AP_CTRL_HS = 0
    AP_CTRL_CHAIN = 1
    AP_CTRL_NONE = 2
    AP_CTRL_ME = 3


class indices (ctypes.Structure):
    _fields_ = [
        ("m_index", ctypes.c_uint16),
        ("m_pc_index", ctypes.c_uint8),
        ("unused", ctypes.c_uint8)
    ]


class ip_u1 (ctypes.Union):
    _fields_ = [
        ("m_base_address", ctypes.c_int64),
        ("indices", indices)
    ]


class ip_data (ctypes.Structure):
    _fields_ = [
        ("m_type", ctypes.c_uint32),
        ("properties", ctypes.c_uint32),
        ("ip_u1", ip_u1),
        ("m_name", ctypes.c_uint8 * 64)
    ]


class ip_layout (ctypes.Structure):
    _fields_ = [
        ("m_count", ctypes.c_int32),
        ("m_ip_data", ip_data*16)
    ]


"""   CONNECTIVITY SECTION   """


class connection(ctypes.Structure):
    _fields_ = [
        ("arg_index", ctypes.c_int32),
        ("m_ip_layout_index", ctypes.c_int32),
        ("mem_data_index", ctypes.c_int32)
    ]


class connectivity(ctypes.Structure):
    _fields_ = [
        ("m_count", ctypes.c_int32),
        ("m_connection", connection*1000)
    ]


cu_args_name = {
    0:  {"name": "dpu_doneclr", "port": "S_AXI_CONTROL"},
    1:  {"name": "dpu_prof_en", "port": "S_AXI_CONTROL"},
    2:  {"name": "dpu_cmd", "port": "S_AXI_CONTROL"},
    3:  {"name": "dpu_instr_addr", "port": "M_AXI_GP0"},
    4:  {"name": "dpu_prof_addr", "port": "M_AXI_GP0"},
    5:  {"name": "dpu_base0_addr", "port": "M_AXI_HP0"},
    6:  {"name": "dpu_base1_addr", "port": "M_AXI_HP0"},
    7:  {"name": "dpu_base2_addr", "port": "M_AXI_HP0"},
    8:  {"name": "dpu_base3_addr", "port": "M_AXI_HP0"},
    9:  {"name": "dpu_base4_addr", "port": "M_AXI_HP2"},
    10: {"name": "dpu_base5_addr", "port": "M_AXI_HP2"},
    11: {"name": "dpu_base6_addr", "port": "M_AXI_HP2"},
    12: {"name": "dpu_base7_addr", "port": "M_AXI_HP2"}
}


def getIPsZynq(path):
    cus = {}
    ip_layout_data = open(path, 'rb').read()
    ips = ip_layout()

    ctypes.memmove(ctypes.pointer(ips), ip_layout_data, len(ip_layout_data))

    for i in range(0, ips.m_count):
        ip = ips.m_ip_data[i]
        cu_name = str()

        for c in ip.m_name:
            cu_name += chr(c)
        cu_type, cu_name = cu_name.strip('\x00').split(':')

        cu_name = cu_name.replace('_xrt_top', '')
        cu_paddr = hex(ip.ip_u1.m_base_address)
        cu_idx = ip.ip_u1.indices.m_index
        cu_pc_idx = ip.ip_u1.indices.m_pc_index

        #cus[cu_name] = (cu_pc_idx,cu_paddr)
        #cus[cu_pc_idx] = (cu_name,cu_paddr)
        cus[i] = (cu_name, cu_paddr)

    return {'cus': cus}


def getConnectivity(path):
    conn_data = open(path, 'rb').read()
    conn = connectivity()

    ret = {}

    ctypes.memmove(ctypes.pointer(conn), conn_data, len(conn_data))
    for i in range(0, conn.m_count):
        con = conn.m_connection[i]
        ret.setdefault(con.m_ip_layout_index, []).append(
            {'arg_index': con.arg_index, 'mem_data_index': con.mem_data_index})
        #print ("m_ip_layout_index %d, arg_index %d, mem_data_index %d" % (con.m_ip_layout_index, con.arg_index, con.mem_data_index))

    return {'connectivity': ret}


def getMemTopo(path):
    mem_topo_data = open(path, 'rb').read()
    mem_topo = mem_topology()

    ret = {}

    ctypes.memmove(ctypes.pointer(mem_topo), mem_topo_data, len(mem_topo_data))

    print(mem_topo.m_count)
    for i in range(mem_topo.m_count):
        print(
            f"{i}: {mem_topo.m_mem_data[i].m_tag}, {mem_topo.m_mem_data[i].m_type}, {mem_topo.m_mem_data[i].mem_u2.flow_id}")
    #xbutil_cmd = "xbutil"
    # for alt in xbutil_path_alt:
    #    if os.path.exists(alt):
    #        xbutil_cmd = alt

    #d = os.popen('%s --legacy dump' % xbutil_cmd).read()
    # if len(d) == 0:
    #    return {}

    #ret = json.loads(d)['board']["memory"]
    # return ret


def checkPath():
    alt_dirs = [
        "/sys/devices/platform/amba/amba:zyxclmm_drm/",
        "/sys/devices/platform/axi/axi:zyxclmm_drm/",
        "/sys/devices/platform/amba_pl@0/amba_pl@0:zyxclmm_drm/"
    ]

    for d in alt_dirs:
        if os.path.isdir(d):
            return d

    logging.error("Cannot find path amba:zyxclmm_drm")
    exit(-1)


path = checkPath()
ip_connect = getConnectivity(os.path.join(path, "connectivity"))
ip_layout = getIPsZynq(os.path.join(path, "ip_layout"))
mem_topo = getMemTopo(os.path.join(path, "mem_topology"))

print(ip_connect)
print(ip_layout)
print(mem_topo)
#
exit(-1)

DPU_TOPO = {}
for dpu_idx in ip_layout['cus'].keys():
    DPU_TOPO.setdefault(dpu_idx, {})['name'] = ip_layout['cus'][dpu_idx][0]
    DPU_TOPO.setdefault(dpu_idx, {})['address'] = ip_layout['cus'][dpu_idx][1]
    DPU_TOPO.setdefault(dpu_idx, {})[
        'ports'] = ip_connect['connectivity'][dpu_idx]
    for port in DPU_TOPO[dpu_idx]['ports']:
        port_idx = port['mem_data_index']
        port_idx_tag = mem_topo['mem'][str(port_idx)]['tag']

        arg_idx = int(port['arg_index'])
        arg_name = cu_args_name[arg_idx]['name']
        port['tag'] = port_idx_tag
        port['arg_name'] = arg_name


def print_dpu_info(dpu_info):

    title = ['CU Index', 'CU Name', 'CU Address',
             'Instruct Port', 'Data Ports']
    datas = []
    datas.append(title)

    for dpu_idx in dpu_info.keys():
        content = dpu_info[dpu_idx]

        cu_id = dpu_idx
        cu_name = content.get("name", "")
        cu_addr = content.get("address", "0x0")
        cu_ports = content.get("ports", [])

        inst_port = [p for p in cu_ports if p["arg_name"] == "dpu_instr_addr"]
        inst_port_tags = str(list(set([p["tag"] for p in inst_port]))).replace(
            "'", "").strip('[').strip(']')

        data_port = [
            p for p in cu_ports if p["arg_name"].find("dpu_base") >= 0]
        data_port_tags = str(list(set([p["tag"] for p in data_port]))).replace(
            "'", "").strip('[').strip(']')

        datas.append([dpu_idx, cu_name, cu_addr,
                      inst_port_tags, data_port_tags])

    print_ascii_table(datas)


print_dpu_info(DPU_TOPO)

#print(json.dumps(DPU_TOPO, indent=1))



#!/usr/bin/python3
# -*- coding:utf-8 -*-

# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ctypes import *
import sys
import os
import time
import threading
import logging
import tracer.tracerBase
import copy

supported_boards = ["zcu102", "zcu104", "vck190", "kv260"]
sys_hw_mon = "/sys/class/hwmon/"


class hwmon_ina226:
    def __init__(self, path, libpath='/usr/lib/libpowermon.so'):
        self.valid = False
        self.board = ""
        self.sample_phases = 1
        self.powerLib = cdll.LoadLibrary(libpath)

        if not os.path.exists(path):
            logging.error("Invalid path [%s]" % path)
            return

        try:
            self.path = os.path.abspath(path)
            self.name = open(os.path.join(path, "name"), "rt").read().strip()
        except:
            logging.error("Invalid path [%s]" % path)
            return

        if self.name.find("ina2") < 0:
            logging.info("Invalid hwmon [%s], name: [%s]" % (path, self.name))
            return

        """
        Fix:
        INA226 for VCCINT monitors output of only one phase of IR35215 regulator. 
        Since VCCINT power rail is supplied by 6 phases of that regulator,
        you would need to multiple INA226 readings by 6
        (that is what sc_app utility do)
        """
        if self.board == "vck190" and self.path.find("hwmon0") >= 0:
            self.sample_phases = 6

        self.v = []
        self.c = []
        self.p = []

        self.volt = 0

        for r, d, files in os.walk(path):
            if os.path.abspath(r) != self.path:
                continue
            for f in files:
                if f.startswith("curr"):
                    self.c.append(os.path.abspath(os.path.join(r, f)))
                if f.startswith("power"):
                    self.p.append(os.path.abspath(os.path.join(r, f)))
                if f.startswith("in"):
                    self.v.append(os.path.abspath(os.path.join(r, f)))

        self.powerLib.get_device_volt.restype = c_int
        self.volt = max([int(self.powerLib.get_device_volt(v.encode()))
                        for v in self.v]) / 1000.0

        if self.volt == 0:
            logging.error("Invalid path [%s]" % path)
            return

        self.valid = True

    def sample_curr(self):
        if self.valid == False:
            return 0
        self.powerLib.get_device_curr.restype = c_int
        curr = max([int(self.powerLib.get_device_curr(c.encode()))
                   for c in self.c]) / 1000.0

        return self.sample_phases * curr

    def sample_power(self):
        if self.valid == False:
            return 0
        self.powerLib.get_device_power.restype = c_int
        power = max([int(self.powerLib.get_device_power(p.encode()))
                    for p in self.p]) / 1000.0 / 1000.0
        return self.sample_phases * power


class PowerTracer(tracer.tracerBase.Tracer):
    def __init__(self):
        super().__init__('power', source=[],
                         compatible={'machine': ["aarch64"]})
        self.mons = []
        self.board = None
        self.power_records = []
        self.power_info = []
        self.started = False
        self.sample_interval = None
        self.sample_thread = threading.Thread(target=self.sample_loop)

    def process(self, data, t_range=[]):
        pass

    def start(self):
        super().start()
        logging.debug("Power Tracer Start")
        self.started = True
        self.sample_thread.start()

    def stop(self):
        super().stop()
        logging.debug("Power Tracer Stop")
        self.started = False
        logging.debug("Power Tracer waiting sample thread stop")
        self.sample_thread.join()

    def checkBoard(self):
        host_name = os.uname().nodename
        # 'xilinx-zcu102-20221'

        for b in supported_boards:
            if host_name.find(b) >= 0:
                self.board = b
        if self.board is None:
            logging.error("Invalid board name: [%s]" % host_name)
            return False

        return True

    def compatible(self, platform: {}):
        if super().compatible(platform) == False:
            return False

        return self.checkBoard()

    def prepare(self, options: dict, debug: bool):
        power_tracer_option = options.get('tracer', {}).get('power', {})
        self.sample_interval = power_tracer_option.get("sample_interval", 0.1)

        logging.info(f"Power tracer sample interval: {self.sample_interval}s")

        for i in range(0, 50):
            mon_path = os.path.join(sys_hw_mon, "hwmon%d" % i)
            if os.path.exists(mon_path):
                mon = hwmon_ina226(mon_path)
                if mon.valid:
                    self.mons.append(mon)

    def getData(self):
        return self.power_records

    def sample_loop(self):
        while self.started:
            power = 0
            for m in self.mons:
                power += m.sample_power()
            self.power_records.append(power)
            time.sleep(self.sample_interval)


tracer.tracerBase.register(PowerTracer())



# -*- coding: UTF-8 -*-

# Copyright 2021 Xilinx Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

VAITRACE_VER = "3.5.0"
VAITRACE_BUILD_DATE = "2023-09-12-23:21:46"
VAITRACE_GIT_VER = "b7953a2a9"
TOP_PROJECT = "vart"

VAITRACE_GIT_INTERNAL_PREV_VER="671d17e"



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import sys
import os
import argparse
import logging

import vaitraceCfgManager
import vaitraceSetting
import vaitraceCppRunner
import vaitracePyRunner
import vaitraceDefaults
import vaitraceWriter


def getVersion():
    head = "vaitrace ver:\n"
    version = ""
    project = ""
    internal_version = ""
    try:
        import vaitrace_version as v
        version = "%s_%s_%s\nProject:\n%s" % \
            (v.VAITRACE_VER, v.VAITRACE_GIT_VER,
             v.VAITRACE_BUILD_DATE, v.TOP_PROJECT)
    except:
        path = os.path.realpath(__file__)
        version = "Dev Version: [%s]" % path
        return version

    try:
        git_inter_prev_ver = v.VAITRACE_GIT_INTERNAL_PREV_VER
    except:
        git_inter_prev_ver = "-"

    try:
        git_inter_ver = v.VAITRACE_INTERNAL_VER
    except:
        git_inter_ver = "-"

    internal_version = "\nInternal Git Ver:\n[%s, %s]" % (
        git_inter_prev_ver, git_inter_ver)

    return head + version + internal_version


def checkPermisson():
    """Checking Permission"""
    if os.getgid() != 0:
        logging.warning("This tool need run as 'root'")
        logging.warning(
            "without 'root' permission, CPU function profiling feature is invalid")


def parseCmdLine():
    """
    -o: trace file: save trace file [default: trace.txt]
    -c: conf: input config file
    -t: time: tracing time in second(s)
    -p: python: for tracing python application
    -v: show version
    """

    default_conf_json = ""
    cmd_parser = argparse.ArgumentParser(prog="vaitrace")
    cmd_parser.add_argument("cmd", nargs=argparse.REMAINDER)
    cmd_parser.add_argument("-c", dest="config", nargs='?',
                            default=default_conf_json, help="Specify the config file")
    cmd_parser.add_argument(
        "-d", dest='debug', action='store_true', help="Enable debug")
    cmd_parser.add_argument("-o", dest="traceSaveTo",
                            nargs='?', help="Save report to, only available for txt summary mode")
    cmd_parser.add_argument("-t", dest='timeout', nargs='?',
                            type=int, help="Tracing time limit in second, default value is 60")
    cmd_parser.add_argument("-v", dest='showversion',
                            action='store_true', help="Show version")
    cmd_parser.add_argument("-b", dest='bypass', action='store_true',
                            help="Bypass vaitrace, just run command")
    cmd_parser.add_argument(
        "-p", dest='python', action='store_true', help="Trace python application")
    cmd_parser.add_argument("--va", dest='va', action='store_true',
                            help="Generate trace data for Vitis Analyzer")
    cmd_parser.add_argument(
        "--xat", dest='xat', action='store_true', help="Save raw data, for debug usage")
    cmd_parser.add_argument(
        "--txt_summary", dest='txt', action='store_true', help="Display txt summary")
    cmd_parser.add_argument(
        "--json_summary", dest='json', action='store_true', help="Display json summary")
    # fine_grained feature was removed by xcompiler since vai-3.0
    #cmd_parser.add_argument(
    #    "--fine_grained", dest='fg', action='store_true', help="Fine grained mode")

    args = cmd_parser.parse_args()

    if args.showversion:
        print(getVersion())
        exit(0)

    return args, cmd_parser


def printHelpExit(parser):
    parser.print_help()
    exit(-1)


def main(args, cmd_parser):

    options = vaitraceDefaults.traceCfgDefaule

    """Configuration priority: Configuration File > Command Line > Default"""
    options['cmdline_args'] = {}
    options['cmdline_args']['cmd'] = args.cmd
    options['cmdline_args']['timeout'] = args.timeout
    options['cmdline_args']['output'] = args.traceSaveTo
    options['cmdline_args']['debug'] = args.debug
    options['cmdline_args']['config'] = args.config
    options['cmdline_args']['bypass'] = args.bypass
    options['cmdline_args']['python'] = args.python

    """
    Select Output Format,
    Default format is vtf (or va), only when --xat exists it will generate .xat"""
    if args.xat == True:
        options['cmdline_args']['xat'] = True
        options['cmdline_args']['va'] = False
        options['cmdline_args']['txt'] = False
        options['cmdline_args']['json'] = False
    elif args.txt == True:
        options['cmdline_args']['txt'] = True
        options['cmdline_args']['xat'] = False
        options['cmdline_args']['va'] = True
        options['cmdline_args']['json'] = False
    elif args.json == True:
        options['cmdline_args']['json'] = True
        options['cmdline_args']['xat'] = False
        options['cmdline_args']['va'] = True
        options['cmdline_args']['txt'] = False
    else:
        options['cmdline_args']['va'] = True
        options['cmdline_args']['xat'] = False
        options['cmdline_args']['txt'] = False

    if args.debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    """
    if args.fg:
        logging.warning("The fine grained profiling model is enabled. Models will be run subgraph-by-subgraph.\n"
                        "1. It will introduce significant scheduling and memory access overhead, especially for small models.\n"
                        "2. Some xcompiler's optimizations will be turned-off.\n")
        options['runmode'] = 'debug'
        options['cmdline_args']['fg'] = True
    """
    logging.debug("Command line args: ")
    logging.debug(args)

    if vaitraceCfgManager.parseCfg(options) == False:
        printHelpExit(cmd_parser)

    vaitraceSetting.setting(options)
    if vaitraceCfgManager.saityCheck(options) == False:
        printHelpExit(cmd_parser)

    if (args.python):
        checkPermisson()
        # Launch Python
        vaitracePyRunner.run(options)
    else:
        checkPermisson()
        # Launch C/C++
        vaitraceCppRunner.run(options)

    vaitraceWriter.write(options)


if __name__ == '__main__':

    main(*parseCmdLine())



#!/usr/bin/python3
# -*- coding: UTF-8 -*-
# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import re
import csv
import sys
import string
import time
import argparse
import json
import signal
import platform
import random
import gzip
from functools import reduce

AISDK_TRACE_VER = "v0.1-190708"
TRACE_NAME_MASK = "__cln2_"
RAW_RECORD_LIMIT = 1000000
fpsCountEvent = "xilinx::ai::DpuTaskImp::run".replace("::", TRACE_NAME_MASK)
cuThreadList = ['zocl-scheduler']

pattFtrace = re.compile(
    r"""
        (?P<taskComm>.+)\-
        (?P<taskPid>\d{1,})\s+\[
        (?P<cpuID>\d{3})\].{4,}\s
        (?P<timeStamp>\d+\.\d{6})\:\s+
        (?P<func>\w+)\:
        (?P<info>.*)
        """,
    re.X | re.M
)

pattAPM = re.compile(
    r'#APM\s(\d+\.\d+)\s(\d+)\s(\d+)\s(\d+)\s(\d+)\s(\d+)\s(\d+)\s(\d+)\s(\d+)\s(\d+)\s(\d+)')

hwCUPatt = re.compile(
    r"""
        .+
        cu_idx=(?P<cu_idx>\d+)
        """,
    re.X | re.M
)

pattSchedSwitch = re.compile(
    r"""
        prev_comm\=(?P<prev_comm>.+)
        prev_pid\=(?P<prev_pid>.+)
        prev_prio\=(?P<prev_prio>.+)
        prev_state\=(?P<prev_state>.+)==>.+
        next_comm\=(?P<next_comm>.+)
        next_pid\=(?P<next_pid>.+)
        next_prio\=(?P<next_prio>.+)
        """,
    re.X | re.M
)

#global T


class traceEvent:
    def __init__(self, _eventName, _timeStamp, _info=None):
        self.event = _eventName
        self.timeStamp = float(_timeStamp)
        self.name, self.postfix = _eventName.rsplit('_', 1)
        self.info = _info

    def __str__(self):
        return "%.6f: %s\n" % (self.timeStamp, self.event)

    def isSched(self):
        return self.event.startswith('sched_')

    def isDpuHw(self):
        return self.event.lower().startswith('cu_')

    def isHw(self):
        return self.isDpuHw()

    def isFtrace(self):
        return self.event.endswith('_entry') or self.event.endswith('_exit')

    def type(self):
        if self.isSched():
            return "sched"
        if self.isDpuHw():
            return "dpuhw"
        else:
            return "ftrace"

    def dir(self):
        if self.isFtrace():
            if self.postfix == 'entry':
                return "in"
            if self.postfix == 'exit':
                return "out"
        elif self.isSched():
            """In sched case, the dir is opposite with normal events"""
            if self.postfix == 'in':
                return "out"
            if self.postfix == 'out':
                return "in"
        elif self.isDpuHw():
            if self.postfix == 'start':
                return "in"
            if self.postfix == 'done':
                return "out"

        print("Event dir err %s" % self, flush=True)

        return "unknown"


class traceRawItem:
    def __init__(self, taskComm, taskPid, cpuId, timeStamp, func, info):
        self.taskComm = taskComm.strip()
        self.taskPid = int(taskPid)
        self.cpuId = int(cpuId)
        self.timeStamp = float(timeStamp)
        self.func = func.strip()
        self.info = info.strip()
        self.infoDetail = dict()
        self.isTarget = False

        if self.isSched():
            if not re.match(pattSchedSwitch, self.info):
                print("Match sched_switch failed", flush=True)
            else:
                infoDetail = re.match(pattSchedSwitch, self.info).groupdict()
                self.infoDetail['ss_prev_comm'] = infoDetail['prev_comm'].strip()
                self.infoDetail['ss_prev_pid'] = int(infoDetail['prev_pid'])
                self.infoDetail['ss_prev_prio'] = int(infoDetail['prev_prio'])
                self.infoDetail['ss_prev_state'] = infoDetail['prev_state'].strip()
                self.infoDetail['ss_next_comm'] = infoDetail['next_comm'].strip()
                self.infoDetail['ss_next_pid'] = int(infoDetail['next_pid'])
                self.infoDetail['ss_next_prio'] = int(infoDetail['next_prio'])

        elif self.isDpuHw():
            if not re.match(hwCUPatt, self.info):
                print("Match hw dpu failed", flush=True)
            else:
                infoDetail = re.match(hwCUPatt, self.info).groupdict()
                self.infoDetail['cu_idx'] = int(infoDetail['cu_idx'].strip())

    def __str__(self):
        return "%.6f:%17s-%4d@[%02d]:%18s: %s" %\
               (self.timeStamp, self.taskComm, self.taskPid,
                self.cpuId, self.func, self.info)

    def isSched(self):
        return self.func.startswith('sched_switch')

    def isCu(self):
        return self.taskComm in cuThreadList or self.func.startswith("cu_")

    def isDpuHw(self):
        return self.func.startswith('cu_')

    def toTraceEvent(self):
        global T
        type = None
        if self.isSched():
            # if self.infoDetail['ss_next_comm'] == 'test_performanc':
            if self.infoDetail['ss_next_comm'] == T.targetComm:
                type = 'sched_switch_in'
            # elif self.infoDetail['ss_prev_comm'] == 'test_performanc':
            elif self.infoDetail['ss_prev_comm'] == T.targetComm:
                type = 'sched_switch_out'
            else:
                print("no used event", flush=True)
                return None
        else:
            type = self.func

        if type is None:
            return None
        return traceEvent(type, self.timeStamp, self.infoDetail)


class traceProcess:
    def __init__(self, startEvent, endEvent):
        self.start = startEvent
        self.end = endEvent
        self.name = startEvent.name
        self.startTime = startEvent.timeStamp
        self.endTime = endEvent.timeStamp
        self.duration = int((self.endTime - self.startTime) * 1000 * 1000)
        self.durationSchedOut = 0
        self.subProc = []
        self.isHw = startEvent.isHw()

    def __str__(self):
        outTime = "" if self.durationSchedOut == 0 else "scheduled out [%sus]" % format(
            self.durationSchedOut, ',')
        hwflag = "[*DPUHW]" if self.isHw else ""

        s = "%s[%s]@[%f-%f]: takes [%sus] %s" % (
            hwflag, self.name, self.startTime, self.endTime, format(self.duration, ','), outTime)
        s = s.replace(TRACE_NAME_MASK, "::")

        return s

    def __lt__(self, other):
        if self.startTime == other.startTime:
            return self.duration > other.duration
        return self.startTime < other.startTime

    def __iter__(self):
        i = iter(self.getAllSubProcs())
        return i

    def getKey(self, _taskPid):
        key = str(_taskPid) + "%s" % str(round(self.startTime * 1000 * 1000))
        return key

    def getAllSubProcs(self):
        result = [self]

        for sub in self.subProc:
            result += sub.getAllSubProcs()

        return result

    def toJsonObj(self, taskPid=0, enableStat=False, taskDuration=1):
        obj = dict()

        #obj['key'] = str(taskPid) + "%s" % str(round(self.startTime * 1000 * 1000))
        obj['key'] = self.getKey(taskPid)
        obj['title'] = self.name.replace(TRACE_NAME_MASK, "::")
        obj['icon'] = False
        obj['durnation'] = self.duration
        obj['durnationSchedOut'] = self.durationSchedOut
        obj['isHw'] = self.isHw

        if not enableStat:
            obj['startTime'] = self.startTime
            obj['endTime'] = self.endTime
        else:
            obj['percentage'] = "%.1f" % ((self.duration / taskDuration) * 100)
            pass

        obj['children'] = [s.toJsonObj(
            taskPid, enableStat, taskDuration) for s in self.subProc]
        if len(obj['children']) > 0:
            obj['folder'] = True

        return obj

    def getRelation(self, subProc):
        rel = "unknown"

        if subProc.startTime < self.startTime:
            print("Wrong order %s-%s" % (self, subProc), flush=True)
            return None

        if subProc.startTime >= self.startTime and subProc.endTime <= self.endTime:
            rel = "child"
        elif subProc.startTime >= self.endTime:
            rel = "sibling"
        else:
            rel = "overlap"
        return rel

    def insertNode(self, subProc):
        for sub in self.subProc:
            if sub.getRelation(subProc) == "child":
                sub.insertNode(subProc)
                return
        self.subProc.append(subProc)

    def getFingerPrint(self):
        fp = hash(self.name)

        for sub in self.subProc:
            fp += sub.getFingerPrint()

        return fp

    # print to console
    def printTree(self, prefix="", file=sys.stdout):
        # print("%s%s"% (prefix, self), file=file)
        prefix = prefix + "----"
        for sub in self.subProc:
            sub.printTree(prefix, file)

    # dump readable result to to memory
    def dumpTree(self):
        pass


class eventParser:
    def __init__(self, parseSched=False, parseDpuHw=True):
        self.parseSched = parseSched

    """Thread paired rule: the same in [name] and opposite in [dir]"""

    def threadPaired(self, startEve, endEve):
        """Error Checking"""
        if startEve.type() != endEve.type():
            return False

        if startEve.timeStamp > endEve.timeStamp:
            return False

        if startEve.dir() != "in" or endEve.dir() != "out":
            return False

        if startEve.isFtrace() or startEve.isSched():
            return startEve.name == endEve.name

        if startEve.isDpuHw():
            return (startEve.name == endEve.name and
                    startEve.info['cu_idx'] == endEve.info['cu_idx'])

        return False

    def parseEvents(self, _events):
        stacks = []
        parseEventResults = []

        """Filter out all sched events if there is no [parseSched]"""
        if not self.parseSched:
            events = [tmp for tmp in _events if tmp.isFtrace()
                      or tmp.isDpuHw()]
        else:
            events = _events

        for e in events:
            stacks.append(e)
            for s in stacks[::-1]:
                if self.threadPaired(s, e):
                    try:
                        stacks.remove(e)
                        stacks.remove(s)
                    except:
                        #print("parse event error", flush=True)
                        pass

                    parseEventResults.append(traceProcess(s, e))

        return sorted(parseEventResults)

    def handleSched(self, rootNode):
        i = 0
        while i < len(rootNode.subProc):
            sub = rootNode.subProc[i]
            if sub.name == "sched_switch":
                if not rootNode.isHw:
                    rootNode.duration -= sub.duration
                    rootNode.durationSchedOut += sub.duration
                sub.duration = 0
                rootNode.subProc.remove(sub)
                i -= 1
            else:
                self.handleSched(sub)
            i += 1

    def parseRoot(self, _events, debug=False):
        proc = self.parseEvents(_events)
        root = proc[0]

        for p in proc[1:]:
            root.insertNode(p)

        if debug:
            root.printTree("|-")

        if self.parseSched:
            self.handleSched(root)

        return root

    def parseCUEvents(self, _events):
        stacks = []
        parseEventResults = []

        events = _events

        for e in events:
            stacks.append(e)
            for s in stacks[::-1]:
                if self.threadPaired(s, e):
                    try:
                        stacks.remove(e)
                        stacks.remove(s)
                    except:
                        #print("parse event error", flush=True)
                        pass

                    parseEventResults.append(traceProcess(s, e))

        return sorted(parseEventResults)

    def cpuEventFilter(self, event):
        if event.isDpuHw():
            return False
        if event.isSched():
            if (event.infoDetail['ss_next_comm'] != T.targetComm or
                    event.infoDetail['ss_prev_comm'] != T.targetComm):
                return False

        return True

    def parseCPUEvents(self, _CPUEvents):
        """Filter out all hardware events"""
        events = _CPUEvents
        schedEvents = [e for e in events if e.isSched()]

        events = [e for e in events if e.isSched()]

        cpuEventsList = []
        idx = 0
        activeTime = 0

        """
        There are three states on CPU:
        ready: sched switched in, but cannot find a named function on running
        run: sched switched in and a named function on running when cpu on this status the color bar will be highlighted
        out: sched switched out
        cpuUtil : cpuUtil = total time - sched switched out time
        """
        state = "out"
        while (idx < len(events) - 1):
            s = events[idx]
            e = events[idx+1]

            "Get a ramdom base color"
            #color = hash(str(s.taskPid)) % (256 * 256 * 256)
            color = (hash(str(s.infoDetail['ss_next_pid'])) >> 2) % (
                256 * 256 * 256)

            # if (s.infoDetail['ss_next_comm'] == T.targetComm):
            #    activeTime += (e.timeStamp - s.timeStamp)
            # else:
            #    color = color | 0xe0e0e0

            if s.isSched():
                if (s.infoDetail['ss_next_comm'] == T.targetComm):
                    state = "ready"
                else:
                    state = "out"
            else:
                if s.taskComm == T.targetComm:
                    state = "run"
                else:
                    state = "out"

            if state == "ready" or state == "run":
                activeTime += (e.timeStamp - s.timeStamp)
            else:
                # if state != "run":
                color = color | 0xe0e0e0

            cpuEventsList.append(
                [s.infoDetail['ss_next_comm'], s.infoDetail['ss_next_pid'], s.timeStamp, e.timeStamp, "#%06x" % color])

            idx = idx+1

        cpuUtil = (activeTime / (T.endTime - T.startTime))

        return cpuEventsList, cpuUtil


class traceThread:
    events = []
    traces = []
    parsed = False
    isCuThread = False

    def __init__(self, _taskComm, _taskPid, _opt, debug=False):
        self.taskComm = _taskComm
        self.taskPid = _taskPid
        self.traces = []
        self.events = []
        self.parsed = False
        self.dump = debug
        self.rootNode = None
        self.options = _opt

    def addItemToThread(self, item):
        self.traces.append(item)
        if item.isCu():
            self.isCuThread = True

    """Add a force end event to the end of records of this thread"""

    def addThreadFlags(self):
        last = self.traces[-1]
        first = self.traces[0]
        forceFirst = traceRawItem(
            "aisdk", "99999", 99, first.timeStamp - 0.00001, "Thread-%d_entry" % self.taskPid, " ")
        forceLast = traceRawItem(
            "aisdk", "99999", 99, last.timeStamp + 0.00001, "Thread-%d_exit" % self.taskPid, " ")
        self.traces.insert(0, forceFirst)
        self.traces.append(forceLast)

    def getTaskPid(self):
        return self.taskPid

    def parse(self):
        print('Analyzing thread: %s-%d' %
              (self.taskComm, self.taskPid), flush=True)

        """Step.1 parse every raw event for this thread """
        for item in self.traces:
            if item.isSched():
                if item.infoDetail['ss_prev_comm'] == self.taskComm and \
                   item.infoDetail['ss_prev_pid'] == self.taskPid:
                    self.events.append(traceEvent(
                        'sched_switch_out', item.timeStamp, item.infoDetail))
                elif item.infoDetail['ss_next_comm'] == self.taskComm and \
                        item.infoDetail['ss_next_pid'] == self.taskPid:
                    self.events.append(traceEvent(
                        'sched_switch_in', item.timeStamp, item.infoDetail))
                continue
            else:
                self.events.append(traceEvent(
                    item.func, item.timeStamp, item.infoDetail))

        """Step.2 divided events into each images"""
        # print(*self.events)
        print("---------------------------------", flush=True)
        p = eventParser(False)
        if self.isCuThread:
            self.rootNode = p.parseCUEvents(self.events)
        else:
            self.rootNode = p.parseRoot(self.events, debug=False)
        self.parsed = True

    def getId(self):
        return "%s-%d" % (self.taskComm, self.taskPid)

    def getTimeline(self):
        if not self.parsed:
            #print("[%s]:Must be parsed at first" % self.getId())
            return None

        title = []
        time = []
        ev = []

        title.append("%s-%d" % (self.taskComm, self.taskPid))

        for e in self.events:
            time.append(e.timeStamp)
            ev.append(e.event)

        return [title, time, ev]


class APMRecord:
    def __init__(self, rec):
        self.time = float(rec[0])
        self.data = []

        for d in rec[1:]:
            self.data.append(int(d)*1000/1024/1024.0)

        self.total = sum(self.data)

    def __str__(self):
        return "Time: %.7f  %.2f" % (self.time, self.total)


class traceTable:
    def __init__(self, _targetComm, _tracePatt, _opt, _title=""):
        self.title = _title
        self.targetComm = _targetComm
        self.targetPid = -1
        self.alivePidTable = []
        self.tracePatt = _tracePatt
        self.threads = []
        self.cpus = [[], [], [], [], [], [], [], [], [], []]
        self.options = _opt
        self.startTime = 1000000000.0
        self.endTime = 0.0
        self.firstRunTime = 9999999999.9
        self.lastRunTime = 0
        self.runCnt = 0
        self.cuUtil = []
        self.cpuUtil = []
        self.cuMap = {}

    def belongsToTarget(self, _pid, _ts):
        return True

    def addRawItems(self,  _items):
        items = _items[0: RAW_RECORD_LIMIT]

        for l in items:
            if not re.match(self.tracePatt, l.strip()):
                continue

            tmp = re.match(self.tracePatt, l.strip()).groups()
            item = traceRawItem(*tmp)

            self.findTarget(item)
            # print(f"Alive Threads' PID: {self.alivePidTable}")

            """Update Time"""
            if item.timeStamp < self.startTime:
                self.startTime = item.timeStamp
            if item.timeStamp > self.endTime:
                self.endTime = item.timeStamp

            #"""Get first and last event time"""
            # if items.index(l) == 0:
            #    self.startTimeStamp = item.timeStamp
            # if items.index(l) == len(items) - 1:
            #    self.endTimeStamp = item.timeStamp

            """Add item to threads"""
            for thread in self.getThread(item):
                if thread is not None:
                    thread.addItemToThread(item)

            """Add item to CPUs"""
            if not item.isCu():
                self.cpus[item.cpuId].append(item)

        for t in self.threads:
            # print(f"{t.taskComm}, {t.taskPid}")
            t.addThreadFlags()

    def updateTs(self, _item):
        pass

    def findTarget(self, _item):
        if not _item.func.startswith('sched_process_'):
            return

        if self.targetComm != None:
            return

        spTs = _item.timeStamp
        spType = _item.func.split('_process_')[1]
        if spType == 'exec':
            execInfoPatt = re.compile(
                r"""
                filename=(?P<filename>.+)\s
                pid=(?P<pid>\d+)\s
                old_pid=(?P<old_pid>\d+)
                """,
                re.X | re.M)
            spInfo = re.match(execInfoPatt, _item.info)
            if (spInfo is not None):
                pid = int(spInfo['pid'])
                self.targetComm = spInfo['filename'].split('/')[-1][:15]
                self.targetPid = pid
                self.alivePidTable.append(pid)
                print("Find Comm %s" % self.targetComm, flush=True)

        elif spType == 'fork':
            forkInfoPatt = re.compile(
                r"""
                comm=(?P<comm>.+)\s
                pid=(?P<pid>\d+)\s
                child_comm=(?P<child_comm>.+)\s
                child_pid=(?P<child_pid>\d+)
                """,
                re.X | re.M)
            spInfo = re.match(forkInfoPatt, _item.info)
            if (spInfo is not None):
                c_pid = int(spInfo['child_pid'])
                if (c_pid in self.alivePidTable):
                    raise ("PID ERROR")
                comm = spInfo['comm']
                print(f"sched_fork: {comm}:{self.targetComm}")
                if self.targetComm == comm:
                    self.alivePidTable.append(c_pid)
                # print(self.alivePidTable)
                #print("[%f]sched_process_%s,pid %s -> [forked] -> %s" % (spTs, spType, spInfo['pid'], spInfo['child_pid']))

        elif spType == 'exit':
            exitInfoPatt = re.compile(
                r"""
                comm=(?P<comm>.+)\s
                pid=(?P<pid>\d+)\s
                prio=(?P<prio>\d+)
                """,
                re.X | re.M)
            spInfo = re.match(exitInfoPatt, _item.info)
            if (spInfo is not None):
                pid = int(spInfo['pid'])
                comm = spInfo['comm']
                print(f"sched_exit: {comm}:{self.targetComm}")
                if self.targetComm == comm:
                    self.alivePidTable.remove(pid)
                    #print("[%f]sched_process_%s, pid %s" % (spTs, spType, spInfo['pid']))
        else:
            pass

    def parseAll(self, opts=None):
        if self.targetComm is None or self.targetComm == "":
            raise RuntimeError("[targetComm] is None")

        for t in self.threads:
            # BUG: should use self.elongsToTarget here
            if t.taskComm.startswith(self.targetComm) or t.isCuThread:
                t.parse()

        self.threads = [t for t in self.threads if t.rootNode is not None]

    """addThread() rerurns the thread(s) this item belongs to, if not matched return none"""

    def getThread(self, item):
        switchOut = None
        switchIn = None

        """sched_switch records will be assigned for 2 times, one for switched in thread, one for switched out thread"""
        if item.isSched():
            for t in self.threads:
                if t.taskPid == item.infoDetail['ss_prev_pid']:
                    switchOut = t
                if t.taskPid == item.infoDetail['ss_next_pid']:
                    switchIn = t
            if not switchIn:
                switchIn = traceThread(
                    item.infoDetail['ss_next_comm'], item.infoDetail['ss_next_pid'], self.options)
                self.threads.append(switchIn)
            if not switchOut:
                switchOut = traceThread(
                    item.infoDetail['ss_prev_comm'], item.infoDetail['ss_prev_pid'], self.options)
                self.threads.append(switchOut)
        else:
            for t in self.threads:
                if t.taskPid == item.taskPid and t.taskComm == item.taskComm:
                    switchIn = t
            if not switchIn:
                self.threads.append(traceThread(
                    item.taskComm, item.taskPid, self.options))
                switchIn = self.threads[-1]

        return switchIn, switchOut

    def printImgView(self, file=sys.stdout):
        for t in self.threads:
            if not t.parsed:
                continue

            #print("[%s ID: %s]" % (prefix, img.getId()), file=file)
            t.rootNode.printTree(prefix='|--', file=file)
            t.rootNode.dumpTree()
            print("", file=file, flush=True)

    def saveImgViewTXT(self, file='image.txt'):
        with open(file, 'w') as f:
            print("AI SDK trace output file name: %s" % file, flush=True)
            self.printImgView(f)

    def getCPUJson(self):
        retData = {}
        p = eventParser(False)

        for i in range(0, len(self.cpus)):
            cpuData, cpuUtil = p.parseCPUEvents(self.cpus[i])
            if cpuUtil > 0:
                coreID = "%d" % i
                util = "%f" % cpuUtil
                T.cpuUtil.append({coreID: util})

            if (len(cpuData) > 0):
                retData['CPU-%d' % i] = cpuData

        return retData

    def getStatisticJson(self):
        stat = {}

        for t in self.threads:
            if t.isCuThread:
                continue
            if (len(t.rootNode.subProc) <= 1):
                continue

            """Filter out the last one"""
            for sub in t.rootNode.subProc[:-1]:
                if sub.name not in stat.keys():
                    stat[sub.name] = []
                stat[sub.name].append(sub)

        def mf(x, y): return dict(x, **y)
        utils = reduce(mf, self.cpuUtil + self.cuUtil)
        #utils["CPU"] = self.cpuUtil
        return {'cpu': 'utilization', 'data': utils}

    def getCUJson(self):
        cuProcs = []
        cuSet = set()
        retData = dict()

        for th in self.threads:
            if not th.isCuThread:
                continue
            for proc in th.rootNode:
                if proc.name == 'cu':
                    cuProcs.append(proc)

        for proc in cuProcs:
            idx = int(proc.start.info['cu_idx'])
            cuSet.add(idx)
            #coreID = "CU-%d" % (idx + list(cuSet).index(idx))
            coreID = "CU-%s" % T.cuMap[hex(idx)]
            startTs = proc.startTime
            endTs = proc.endTime

            if coreID not in retData.keys():
                retData[coreID] = []
            retData[coreID].append((startTs, endTs))

        for id in retData.keys():
            retData[id].sort(key=lambda x: x[0])
            totalDelta = retData[id][-1][1] - retData[id][0][0]
            runTime = 0
            for i in retData[id]:
                runTime += i[1] - i[0]
            util = "%.1f%%" % (runTime / totalDelta * 100)
            latency = runTime * 1000 / len(retData[id])
            util += "|Task Lat: %.1fms" % latency
            """Format CU-ID: [CU-2147745792-->CU@0x80040000] """
            if (id.startswith('CU-')):
                """Last 4-bit represent CU index"""
                addr = T.cuMap[id.split('-')[1]]
                #addr = hex((int(id.split('-')[1]) >> 4) << 4)
                #idx = int(id.split('-')[1]) & 0xf
                idx = T.cuMap[addr]
                id = "CU-%s@%s" % (idx, addr)

            self.cuUtil.append({id: util})

        return retData

    def getThreadJson(self):
        retData = dict()

        for t in self.threads:
            if (len(t.rootNode.subProc) == 0):
                continue
            retData['Thread-%d' % t.taskPid] = []
            for s in t.rootNode.subProc:
                #print (s.startTime, s.endTime)
                retData['Thread-%d' % t.taskPid].append(
                    (s.startTime, s.endTime, s.name.replace(TRACE_NAME_MASK, "::")))

        return retData


def sched_parser_main(src):

    if len(src) == 0:
        return {}

    options = {'thread_view': True, 'trace_sched_switch': False}
    data = {}

    global T
    T = traceTable(None, pattFtrace, options)
    T.addRawItems(src)
    T.parseAll()

    #targetFile = sys.argv[2]

    # with open(targetFile, "w+") as f:
    # data.update(T.getCUJson())

    """Get CPU data"""
    data.update(T.getCPUJson())

    """Get statstic info"""
    # data.update(T.getStatisticJson())
    stat = T.getStatisticJson()

    # print(stat)
    # utilization
    #f.write(json.dumps(stat, indent=1, sort_keys=True))
    ret = {"cpu": {"utilization": stat['data']}}

    return ret


if __name__ == '__main__':
    f = open(sys.argv[1], 'rt')

    source = f.readlines()
    sched_parser_main(source)
    f.close()




# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from vaitrace_py import vai_tracepoint
import os

"""
export VAI_TRACE_ENABLE=1
export VAI_TRACE_TS=XRT
"""
os.environ.setdefault("VAI_TRACE_ENABLE", "1")
os.environ.setdefault("VAI_TRACE_TS", "XRT")
os.environ.setdefault("VAI_TRACE_DIR", "./")


@vai_tracepoint
def hello_vaitrace(a, b):
    return a+b


@vai_tracepoint
def okey_dokey(a, b):
    return a*b


for i in range(100):
    if i % 2 == 0:
        hello_vaitrace(i*5, 555)
    else:
        okey_dokey(i, i*3)




# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import time


class Tracer:
    def __init__(self, name: str, source, compatible):
        self.name = name
        self.timesync = 0
        self.compatibleList = compatible
        self.enabled = False

        if type(source) is str:
            self.source = [source]
        elif type(source) is list:
            self.source = source
        else:
            assert ()

    def compatible(self, platform: {}):
        machine = platform['machine']
        release = platform['release']

        if machine in self.compatibleList['machine']:
            logging.debug("Compatible [%s]: True" % self.name)
            return True

        logging.debug("Compatible [%s]: False" % self.name)
        return False

    def prepare(self, options, debug=False):
        return options

    def start(self):
        self.running = True

    def stop(self):
        self.running = False

    def enable(self):
        self.enabled = True

    def disable(self):
        self.enabled = False

    def process(self, data, t_range=[]):
        pass

    def getData(self):
        pass


__tracers = []
__activeTracers = []


def getTracerInstances() -> list:
    return __tracers


def getAvailableTracers() -> list:
    return [t.name for t in __tracers]


def getSourceRequirement() -> list:
    req = []

    for t in __tracers:
        if type(t.source) is str:
            req.append(t.source)
        elif type(t.source) is list:
            req = req + t.source
        else:
            assert (True)

    return req


def start():
    global __start_time

    __start_time = time.monotonic()
    for t in __tracers:
        if t.enabled == False:
            continue
        t.start()


def stop():
    global __stop_time

    __stop_time = time.monotonic()
    for t in __tracers:
        if t.enabled == False:
            continue
        t.stop()


def process(data: dict):
    for t in __tracers:
        if t.enabled == False:
            continue
        logging.debug("Processing [%s]..." % t.name)
        t.process(data, [__start_time, __stop_time])


def prepare(options: dict) -> dict:
    debug = options['control'].get('debug', False)

    def merge(a: dict, b: dict):
        if hasattr(a, "keys") and hasattr(b, "keys"):
            for kb in b.keys():
                if kb in a.keys():
                    merge(a[kb], b[kb])
                else:
                    a.update(b)

    for t in __tracers:
        logging.debug("Preparing [%s]..." % t.name)
        platform = options['control']['platform']

        disable = options.get('tracer', {}).get(
            t.name, {}).get("disable", False)
        if disable:
            continue

        if t.compatible(platform):
            t.enable()
            merge(options, t.prepare(options, debug))

    return options


def getData():
    data = {}
    for t in __tracers:
        if t.enabled == False:
            continue
        logging.debug("Getting Data [%s]..." % t.name)
        data.update({t.name: t.getData()})
        data.setdefault("timesync", {}).update({t.name: t.timesync})
        data.setdefault("timesync", {}).update(
            {"time_range": [__start_time, __stop_time]})
        logging.debug("[time sync]: %s: %f" % (t.name, t.timesync))

    return data


def register(tracerInstance):
    __tracers.append(tracerInstance)



#!/usr/bin/python3
# -*- coding:utf-8 -*-

# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from ctypes import *
import logging

import tracer.tracerBase
import copy
import os

idcode_ddrmc_num_dict = {'0x4c9b093':3, '0x4c93093':2, '0x4c98093':3, '0x4ca9093':4, '0x4ca8093':4, '0x4cd2093':3, '0x4cd0093':3,
         '0x4c9a093':3, '0x4cc1093':1, '0x4cc0093':1, '0x4cc9093':1, '0x4cc8093':1, '0x4cd2093':3, '0x4cd3093':3}
class APMRecord(Structure):
    _fields_ = [('time', c_double), ('data', c_ulonglong * 10)]

    def output(self):
        return "APM %.7f %d %d %d %d %d %d %d %d %d %d\n" % (self.time, *self.data)


class APM:
    def __init__(self, mem_type, devid, path='/usr/lib/libmemperf.so', enable=True):
        self.apmLib = cdll.LoadLibrary(path)
        self.data = []
        self.base_addr = []
        self.interval = 0
        self.act_period = 0
        self.record_data_len = 0
        self.type = mem_type
        self.enabled = enable
        if mem_type == "noc":
            self.base_addr = self.get_noc_base_addr(self.type, devid)
            if self.enabled == False:
                return
            self.noc_base_addr = (c_int * len(self.base_addr))(*self.base_addr)
            self.apmLib.create_noc_instance(
                    self.type.encode(), self.noc_base_addr, int(len(self.base_addr)))
        elif mem_type == "apm":
            self.apmLib.create_apm_instance(self.type.encode())

    def start(self, interval=1.0):
        if self.enabled == False:
            return
        self.interval = interval
        self.apmLib.start.restype = c_double
        return self.apmLib.start(c_double(interval))

    def pushData(self, data):
        if self.enabled == False:
            return
        for i in range(0, len(data.data)):
            data.data[i] = data.data[i]

        self.data.append(copy.deepcopy(data))

    def printData(self, data):
        if self.enabled == False:
            return
        info = str()
        info += "TimeStamp: %.7f\n" % data.time

        read = [data.data[i]/self.act_period/1024 /
                1024 for i in range(0, 10) if i % 2 == 0]
        info += "Read Ports:  "
        for d in read:
            info += "%8.1f" % d

        info += " MB/s\n"

        write = [data.data[i]/self.act_period/1024 /
                 1024 for i in range(0, 10) if i % 2 == 1]
        info += "Write Ports: "
        for d in write:
            info += "%8.1f" % d

        info += " MB/s\n  "

        return info

    def get_noc_base_addr(self, mem_type, devid):
        if mem_type != "noc":
            return
        ddrmc_number = idcode_ddrmc_num_dict.get(devid, 0)
        if ddrmc_number == 0:
            logging.info("Can not get ddrmc/noc number!")
            self.enabled = False
            return
        pre_base_addr = []
        for (dirpath, dirnames, filenames) in os.walk("/proc/device-tree/axi"):
            for file in dirnames:
                if file.startswith("memory-controller"):
                    compatible = open(os.path.join(os.path.abspath(os.path.join(
                        dirpath, file)), "compatible"), "rt").read().strip().split(',')
                    driver_name = compatible[1].split('-')
                    if driver_name[1] != 'ddrmc':
                        continue
                    reg = os.path.join(os.path.abspath(
                        os.path.join(dirpath, file)), "reg")
                    f = open(reg, "rb")
                    numl = list(f.read())
                    addr_off = ((numl[20] << 24) + (numl[21] <<
                                16) + (numl[22] << 8) + (numl[23] << 0))
                    pre_base_addr.append(int(addr_off))

        pre_base_addr.sort()
        return pre_base_addr[0:ddrmc_number]

    def stop(self):
        if self.enabled == False:
            return
        self.apmLib.stop()

        data = APMRecord()
        pd = pointer(data)

        while (self.apmLib.pop_data(pd) == 0):
            self.data.append(copy.deepcopy(data))

        self.apmLib.get_act_period.restype = c_double
        self.act_period = self.apmLib.get_act_period()

        self.apmLib.get_record_data_len.restype = c_int
        self.record_data_len = self.apmLib.get_record_data_len()

    def transTimebase(self):
        if self.enabled == False:
            return
        for i in range(0, len(self.data)):
            for j in range(0, self.record_data_len):
                self.data[i].data[j] = int(
                    self.data[i].data[j] / self.act_period)


def checkAPM():
    if os.path.exists("/dev/uio1") == False:
        return False

    """/sys/class/uio/uio1/device/of_node/name"""
    if os.path.exists("/sys/class/uio/uio1/device/of_node/name") == False:
        return False

    with open("/sys/class/uio/uio1/device/of_node/name", "rt") as f:
        try:
            name = f.read().strip()
            if name.startswith('perf-monitor') == False:
                return False
        except:
            return False

    return True


class xapmTracer(tracer.tracerBase.Tracer):
    def __init__(self):
        super().__init__('xapm', source=[],
                         compatible={'machine': ["aarch64"]})
        self.apm = None
        self.apm_type = None
        self.devid = None
        self.apm_json_enable = None
        self.sample_base_addr = []

    def prepare(self, option: dict, debug: bool):
        "Handle Input Options"
        xapmOption = option.get('tracer', {}).get('xapm', {})
        self.interval = xapmOption.get("APM_interval", 0.002)
        self.apm = APM(self.apm_type, self.devid)
        if self.apm_type == "noc":
            self.sample_base_addr = self.apm.base_addr
            option.update({"ddrmc_base_addr": self.sample_base_addr})
        "Handle Output Options"
        self.apm_json_enable = option.get('tracer', {}).get('noc_perf', {}).get('ddrmc_json_enable', {})
        option['ddrmc_json_enable'] = self.apm_json_enable
        return option

    def start(self):
        super().start()
        # self.timesync = self.apm.start(self.interval)
        self.apm.start(self.interval)

    def stop(self):
        super().stop()
        self.apm.stop()

    def process(self, data, t_range=[]):
        self.apm.transTimebase()

    def compatible(self, platform: {}):
        if super().compatible(platform) == False:
            return False

        if platform.get('model').startswith('xlnx,zocl-versal'):
            self.apm_type = "noc"
            self.devid = '0x' + platform.get('idcode')[-7:]
            return True
        elif platform.get('model').startswith('xlnx,zocl'):
            self.apm_type = "apm"
            return checkAPM()
        else:
            return False

    def getData(self):
        if self.apm.enabled == False:
            return
        return [d.output() for d in self.apm.data]


if __name__ == '__main__':
    apm = APM()
    apm.start(0.001)
    time.sleep(1)
    apm.stop()

    for a in apm.data:
        print(apm.printData(a))

else:
    tracer.tracerBase.register(xapmTracer())



# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

PREC = 3
TINY_FLG = "~0"


def to_Ki(_num):
    return _num / 1000.0


def to_Mi(_num):
    return _num / 1000.0 / 1000.0


def to_Gi(_num):
    return _num / 1000.0 / 1000.0 / 1000.0


def to_KB(_num):
    return _num / 1000.0


def to_MB(_num):
    return _num / 1000.0 / 1000.0


def to_GB(_num):
    return _num / 1000.0 / 1000.0 / 1000.0


"""
output_type: float/int/str
unit: Ki/Mi/Gi/KB/MB/GB

"""


class uConv:
    def __init__(self, _raw_num, _unit="", _prec=PREC):
        self.num = _raw_num
        self.o_num = self.num
        self.prec = _prec
        self.min = 10**(-_prec)

        _unit = _unit.lower()
        self.unit = _unit

        if _unit == "ki":
            self.o_num = float(self.num) / 10**3
        elif _unit == "mi":
            self.o_num = float(self.num) / 10**6
        elif _unit == "gi":
            self.o_num = float(self.num) / 10**9
        elif _unit == "kb":
            self.o_num = float(self.num) / 2**10
        elif _unit == "mb":
            self.o_num = float(self.num) / 2**20
        elif _unit == "gb":
            self.o_num = float(self.num) / 2**30
        elif _unit == "%":
            self.o_num = float(self.num) * 100
        else:
            pass

    def __str__(self):
        if self.o_num < self.min:
            return TINY_FLG
        else:
            fmt = "{:.%df}" % self.prec
            return fmt.format(self.o_num)

    def __float__(self):
        return self.o_num

    def __int__(self):
        return int(o_num)

    def __format__(self, code):
        if code == "" or code.endswith('s'):
            return self.__str__()

        fmt = "%%%s" % code
        return fmt % self.o_num




# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import sys
import os
import json
import tracer.tracerBase


class xrtTracer(tracer.tracerBase.Tracer):
    def __init__(self):
        super().__init__('xrt', source=[], compatible={
            'machine': ["x86_64", "aarch64"]})
        self.xrtInfo = {}

    """
	$ xbutil scan
	INFO: Found total 1 card(s), 1 are usable
	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	System Configuration
	OS name:        Linux
	Release:        3.10.0-957.el7.x86_64
	Version:        #1 SMP Thu Nov 8 23:39:32 UTC 2018
	Machine:        x86_64
	Model:          PowerEdge R740
	CPU cores:      12
	Memory:         46525 MB
	Glibc:          2.17
	Distribution:   CentOS Linux 7 (Core)
	Now:            Thu Sep 10 15:51:04 2020
	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	XRT Information
	Version:        2.6.655
	Git Hash:       2d6bfe4ce91051d4e5b499d38fc493586dd4859a
	Git Branch:     2020.1
	Build Date:     2020-05-22 19:05:52
	XOCL:           2.6.655,2d6bfe4ce91051d4e5b499d38fc493586dd4859a
	XCLMGMT:        2.6.655,2d6bfe4ce91051d4e5b499d38fc493586dd4859a
	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     [0] 0000:3b:00.1 xilinx_u50_gen3x4_xdma_base_2 user(inst=128)

    """

    def prepare(self, option: dict, debug: bool):
        return {}

    def process(self, data, t_range=[]):
        """
        Turn off all XRT profile and trace options
        So that, we can prevent 'xbutil dump' generates another xclbin.ex.run_summary that would
        over-write the prev one for vitis-ai process
        """
        os.environ["Debug.profile"] = "false"
        os.environ["Debug.xrt_profile"] = "false"
        os.environ["Debug.vitis_ai_profile"] = "false"
        os.environ["Debug.lop_trace"] = "false"
        os.environ["Debug.timeline_trace"] = "false"
        os.environ["Debug.data_transfer_trace"] = "off"

        xbutil_path_alt = ["/usr/bin/xbutil", "/opt/xilinx/xrt/bin/xbutil"]
        xbutil_cmd = "xbutil"
        for alt in xbutil_path_alt:
            if os.path.exists(alt):
                xbutil_cmd = alt

        try:
            d = os.popen('%s dump' % xbutil_cmd).read()

            if len(d) == 0:
                return

            self.xrtInfo = json.loads(d)
        except:
            return

    def getData(self):
        return self.xrtInfo


tracer.tracerBase.register(xrtTracer())



#!/usr/bin/python3
# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ctypes import *
import sys
import os
import time
import atexit
import copy

available_boards = ["zcu102", "zcu104", "vck190", "kv260"]


class hwmon_ina226:
    def __init__(self, path, libpath='/usr/lib/libpowermon.so'):
        self.valid = False
        self.board = ""
        self.sample_phases = 1
        self.powerLib = cdll.LoadLibrary(libpath)

        if not os.path.exists(path):
            print("Err: Invalid path [%s]" % path)
            return

        try:
            self.path = os.path.abspath(path)
            self.name = open(os.path.join(path, "name"), "rt").read().strip()
        except:
            print("Err: Invalid path [%s]" % path)
            return

        if self.name.find("ina2") < 0:
            print("Err: Invalid hwmon [%s], name: [%s]" % (path, self.name))
            return

        # check_board
        host_name = os.uname().nodename
        # 'xilinx-zcu102-20221'

        for b in available_boards:
            if host_name.find(b) >= 0:
                self.board = b
        if self.board == "":
            print("Err: Invalid board name: [%s]" % host_name)
            return

        """
        Fix:
        INA226 for VCCINT monitors output of only one phase of IR35215 regulator. 
        Since VCCINT power rail is supplied by 6 phases of that regulator,
        you would need to multiple INA226 readings by 6
        (that is what sc_app utility do)
        """
        if self.board == "vck190" and self.path.find("hwmon0") >= 0:
            self.sample_phases = 6

        self.v = []
        self.c = []
        self.p = []

        self.volt = 0

        for r, d, files in os.walk(path):
            if os.path.abspath(r) != self.path:
                continue
            for f in files:
                if f.startswith("curr"):
                    self.c.append(os.path.abspath(os.path.join(r, f)))
                if f.startswith("power"):
                    self.p.append(os.path.abspath(os.path.join(r, f)))
                if f.startswith("in"):
                    self.v.append(os.path.abspath(os.path.join(r, f)))

        self.powerLib.get_device_volt.restype = c_int
        self.volt = max([int(self.powerLib.get_device_volt(v.encode()))
                        for v in self.v]) / 1000.0

        if self.volt == 0:
            print("Err: Invalid path [%s]" % path)
            return

        self.valid = True

    def sample_curr(self):
        if self.valid == False:
            return 0
        self.powerLib.get_device_curr.restype = c_int
        curr = max([int(self.powerLib.get_device_curr(c.encode()))
                   for c in self.c]) / 1000.0
        return self.sample_phases * curr

    def sample_power(self):
        if self.valid == False:
            return 0
        self.powerLib.get_device_power.restype = c_int
        power = max([int(self.powerLib.get_device_power(p.encode()))
                    for p in self.p]) / 1000.0 / 1000.0
        return self.sample_phases * power


mons = []
power_records = []
sys_hw_mon = "/sys/class/hwmon/"


def print_statistics():
    idle_power = min(power_records)
    max_power = max(power_records)
    ave_power = sum(power_records) / len(power_records)

    report_fmt = "Power Statistics: Idle {:.3f} w, Peak {:.3f} w, Average {:.3f} w"
    report_str = report_fmt.format(idle_power, max_power, ave_power)
    report_separator = "=" * len(report_str)

    print("\n\n{}\n{}\n{}".format(report_separator, report_str, report_separator))


for i in range(0, 100):
    mon_path = os.path.join(sys_hw_mon, "hwmon%d" % i)
    if os.path.exists(mon_path):
        mon = hwmon_ina226(mon_path)
        if mon.valid:
            mons.append(mon)

if __name__ == "__main__":
    try:
        while True:
            power = 0

            for m in mons:
                #print("%5.4f" % (m.sample_power()), end = " " )
                power += m.sample_power()

            print("%.3f" % power)
            print("-" * 10)
            power_records.append(power)
            time.sleep(0.2)
    except KeyboardInterrupt:
        print_statistics()



# Copyright 2022-2023 Advanced Micro Devices Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import os
from ctypes import *


class InstDumpHeader(LittleEndianStructure):
    _pack_ = 1
    _fields_ = [
        ('bank_addr', c_uint, 13),
        ('output_channel_num', c_uint, 7),
        ('dpby', c_uint, 4),
        ('dpdon', c_uint, 4),
        ('opcode', c_uint, 4),


        ('jump_read', c_uint, 16),
        ('bank_id', c_uint, 8),
        ('ddr_mode', c_uint, 1),
        ('pad_idx', c_uint, 5),
        ('r0', c_uint, 2),

        ('channel', c_uint, 12),
        ('mode_avg', c_uint, 2),
        ('length', c_uint, 10),
        ('jump_write', c_uint, 8),

        ('ddr_addr', c_uint, 29),
        ('reg_id', c_uint, 3),

        ('jump_read_endl', c_uint, 21),
        ('pad_end', c_uint, 6),
        ('pad_start', c_uint, 5)
    ]


class InstDumpItem(LittleEndianStructure):
    _pack_ = 1
    _fields_ = [
        ('bank_addr', c_uint, 13),
        ('r0', c_uint, 7),
        ('dpby', c_uint, 4),
        ('dpdon', c_uint, 4),
        ('opcode', c_uint, 4),

        ('jump_write', c_uint, 16),
        ('bank_id', c_uint, 8),
        ('r1', c_uint, 8),

        ('channel', c_uint, 12),
        ('r2', c_uint, 2),
        ('length', c_uint, 10),
        ('jump_read', c_uint, 8),

        ('ddr_addr', c_uint, 29),
        ('reg_id', c_uint, 3)
    ]


data = open("./profiler__batch_0_instr_all_start_0.bin", "rb").read()
